#!/usr/bin/python

import glob, os, re, time, argparse, urllib2, zipfile, shutil, ntpath
from os.path import expanduser, join
from urlparse import urlparse

# to set terminal colors when executing this script
class bcolors:
    HEADER =  '\033[95m'
    OKBLUE =  '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL =    '\033[91m'
    ENDC =    '\033[0m'

# Check if online zipfile exists
def test_if_url_exists(path):
    resource = urllib2.urlopen(path)
    status = resource.getcode()
    return status

# Set output directory to merge files in to
def set_output_directory(export, dir_workspace, dir_animate):
    if export is None:
        return join(dir_workspace, dir_animate, 'media')
    else:
        return export

# Set location of zipfile
def set_zipfile_location(path, download_location, zip_fname):
    o = urlparse(path)
    if o.scheme == 'http' or o.scheme == 'https':
        if test_if_url_exists(path) == 200:
            return path
        else:
            return download_location
    else:
        if 'master.zip' not in path:
            return os.path.join(path, zip_fname)
        else:
            return path

# Extract the archive for processing
def extract_to_temp_directory(temp_outdir, archive):
    with zipfile.ZipFile(archive, 'r') as zf:
        zf.extractall(temp_outdir)
        return True

# Create temporary directory
def create_temp_dir(temp_dir):
    if os.path.exists(temp_dir):
        shutil.rmtree(temp_dir)
        os.makedirs(temp_dir)
    else:
        os.makedirs(temp_dir)
    return temp_dir

# If needed download archive
def download_archive_if_needed(temp_dir, archive, zip_fname):
    o = urlparse(archive)
    if o.scheme == 'http' or o.scheme == 'https':
        remote = archive
        local = join(temp_dir, zip_fname)

        u = urllib2.urlopen(remote)
        h = u.info()
        totalSize = int(h["Content-Length"])

        print "Downloading %s bytes ..." % totalSize,
        fp = open(local, 'wb')

        blockSize = 8192 #100000 # urllib.urlretrieve uses 8192
        count = 0
        while True:
            chunk = u.read(blockSize)
            if not chunk: break
            fp.write(chunk)
            count += 1
            if totalSize > 0:
                percent = int(count * blockSize * 100 / totalSize)
                if percent > 100: percent = 100
                print "%2d%%" % percent,
                if percent < 100:
                    print "\b\b\b\b\b",  # Erase "NN% "
                else:
                    print "Done."

        fp.flush()
        fp.close()
        if not totalSize:
            print
    else:
        local = join(temp_dir, archive)
    return local

# Set the properties of the library
def set_properties_file(source_file, out_file, times=None):
    if os.path.exists(out_file):
        os.remove(out_file)
    fhandle = open(out_file, 'a')
    try:
        os.utime(out_file, times)

        with open(source_file, "r") as r:
            with open(out_file, "a") as o:
                o.write('/* Original file: ' + file_path(source_file) + ' added on: ' + time.strftime("%c") + ' download at: https://github.com/geoffgraham/animate.scss */\n')
                o.write(r.read() + '\n')

                f_length = len(file_path(source_file))
                space = ' '

                if f_length < 25:
                    f_length = 25 - f_length
                    space = space * f_length

                print '  ' + bcolors.OKGREEN + '+' + bcolors.ENDC + ' added content of "' + chomp(file_path(source_file)) + '" ' + space + ' > ' + bcolors.OKBLUE + file_path(out_file) + bcolors.ENDC

    finally:
        fhandle.close()
    #return properties_file

# Create list of directories used for creating the final file names
def set_file_names(temp_outdir, extracted_dir, export_dir):
    animate_scss_dir_list = []
    for f in os.listdir(join(temp_outdir, extracted_dir)):
        if re.match(r'_', f) and not re.match(r'_properties.scss', f):
            animate_scss_dir_list.append(f)
        elif re.match(r'_properties.scss', f):

            source_file = join(temp_outdir, extracted_dir, '_properties.scss')
            out_file = join(export_dir, 'animate_properties-debug.scss')
            set_properties_file(source_file, out_file)
    return animate_scss_dir_list

# Chomp newline chars from list
def chomp(s):
    return s.rstrip('\n')

# Shorten path of file names
def file_path(path):
    head, tail = ntpath.split(path)
    return tail or ntpath.basename(head)

# Create files in export directory
def generate_files_in_export_dir(temp_outdir, export_dir, file_names_list, times=None):
    for dname in file_names_list:
        file_name = join(export_dir, 'animate' + dname + '-debug.scss')
        if os.path.exists(file_name):
            os.remove(file_name)
        fhandle = open(file_name, 'a')

        print 'Generated file "' + bcolors.OKBLUE + file_path(file_name) + bcolors.ENDC + '" [' + bcolors.OKGREEN + 'DONE' + bcolors.ENDC + ']'

        for source_file in glob.glob(join(temp_outdir, 'animate.scss-master', dname, '_*.scss')):

            try:
                os.utime(file_name, times)

                with open(source_file, "r") as r:
                    with open(file_name, "a") as o:
                        o.write('/* Original file: ' + file_path(source_file) + ' added on: ' + time.strftime("%c") + ' download at: https://github.com/geoffgraham/animate.scss */\n')
                        o.write(r.read() + '\n')

                        f_length = len(file_path(source_file))
                        space = ' '

                        if f_length < 25:
                            f_length = 25 - f_length
                            space = space * f_length

                        print '  ' + bcolors.OKGREEN + '+' + bcolors.ENDC + ' added content of "' + chomp(file_path(source_file)) + '" ' + space + ' > ' + bcolors.OKBLUE + file_path(file_name) + bcolors.ENDC

            finally:
                fhandle.close()
    return True

# Remove temporary directory on exit
def temporary_dir_cleanup(temp_outdir):
    shutil.rmtree(temp_outdir)
    return True

# Main program to execute functions
def main(path, export=False):

    # home dir
    home = expanduser("~")

    # Set workspace of project
    dir_workspace = home + '/workspace/iqportal/'

    # Set animate directory
    dir_animate = 'basesoft/web/contrib/animate/'

    # Directory this program is running from
    current_dir = os.getcwd()

    # Download location of animate.scss
    download_location = "https://github.com/geoffgraham/animate.scss/archive/master.zip"

    # Set name of temporary zipfile
    zip_fname = 'animate.scss-master.zip'

    # Create temp directory to extract zipfile into
    temp_dir = "%s/animate/" % current_dir

    # Set export directory in workspace tree if not given -e --export option
    export_dir = set_output_directory(export, dir_workspace, dir_animate)

    # Set location of zipfile archive to process (local or online)
    archive = set_zipfile_location(path, download_location, zip_fname)

    # Create temp directory
    temp_outdir = create_temp_dir(temp_dir)

    # If needed download archive
    archive_location = download_archive_if_needed(temp_outdir, archive, zip_fname)

    # Extract zipfile archive to temporary directory
    extract_archive = extract_to_temp_directory(temp_outdir, archive_location)

    # Create list of directories used for creating the final file names
    file_names_list = set_file_names(temp_outdir, 'animate.scss-master', export_dir)

    # Create files in export directory
    create_files = generate_files_in_export_dir(temp_outdir, export_dir, file_names_list)

    temporary_dir_cleanup(temp_outdir)

if __name__ == "__main__":

    # Set arguments to terminal command and Help
    parser = argparse.ArgumentParser(
        prog='./add_animate_scss_library',
        description='''This tool is to extract and install the animate.scss zipfile
        (found at: https://github.com/geoffgraham/animate.scss/archive/master.zip)
        to the animate package tree of IQPortal.'''
    )
    parser.add_argument(
        '-p','--path',
        dest='path',
        help='''Path to local or URI location of animate.scss-master.zip file to use.''',
        required=True
    )
    parser.add_argument(
        '-e','--export',
        dest='export',
        help='''Path to animate package tree in IQPortal to extract files to.''',
        required=False
    )
    args = parser.parse_args()

    main(args.path, args.export)
