#!/bin/bash

#************************************************
# INSTRUCTIONS:
# Run script as user, e.g:
# ./related/bin/add_fontello_fonts.sh
#************************************************

# To be processed directories and files
download_dir="$HOME/Downloads"
font_dir="$download_dir/fontello/"
font_filename="fontello"
local_dir="/usr/share/fonts/truetype/fontello/"
workspace_dir="$HOME/workspace/iqportal/basesoft/web/core/media/fonts/fontello/"
json_dir="$download_dir/fontello/"
json_file="config.json"

# green|all DONE sign
green_done=($(tput bold)$(tput setaf 2)DONE$(tput sgr0))
all_done=($(tput bold)$(tput setaf 3)\<ALL DONNER\>$(tput sgr0))

# Commands to execute
command_cp_font_to_lcl_dir=(sudo cp $font_dir$font_filename.ttf $local_dir)
listing_of_local_dir=(stat --printf='%w %z - %n\n' $local_dir$font_filename.ttf)
get_latest_fontello_zip_file=$(ls -t $download_dir | grep -i "fontello-" | head -1)
mkdir_fontello_in_downloads=(mkdir -p $download_dir/fontello)
unzip_fontello_zip_file_to_fontello_dir=(unzip -oj $download_dir/$get_latest_fontello_zip_file -d $json_dir)
edit_demo_html_file=(sudo sed -i 's/css\///' $json_dir/demo.html)
edit_fontello_css_file=(sudo sed -i 's/\.\.\/font\///' $json_dir/fontello.css)

command_cp_font_to_workspace=(cp $font_dir$font_filename.eot $font_dir$font_filename.svg $font_dir$font_filename.ttf $font_dir$font_filename.woff $workspace_dir)
listing_of_workspace_dir=(stat --printf='%w %z - %n\n'  $workspace_dir$font_filename.eot $workspace_dir$font_filename.svg $workspace_dir$font_filename.ttf $workspace_dir$font_filename.woff)

command_flush_cache=(sudo fc-cache -fs)

command_cp_json_file_tp_workspace=(cp $json_dir$json_file $workspace_dir)
listing_of_json_file=(stat --printf='%w %z - %n\n' $workspace_dir$json_file)

clear

function create_if_fontello_dir_doesnt_exists {
	if [ ! -d "$font_dir" ]; then
        # create font directory in Downloads
        echo 'Created directory:' $json_dir
        "${mkdir_fontello_in_downloads[@]}"
	fi
}

function check_if_fontello_zip_file_exists {
	#check for latest fontello-NUMBER/CHARS.zip file
	if [ ! -f $download_dir/$get_latest_fontello_zip_file ]; then
        #exit with error : zip file not found, download file from fontello.com website
        echo
        echo
        echo 'Please, download fontello fonts at http://www.fontello.com to your Downloads directory.'
        echo -e '\t' 'example:' $HOME/Downloads/fontello-d0e5af8e.zip
        echo
        echo $(tput bold)$(tput setab 1)'IMPORTANT NOTE'$(tput sgr0)
        echo 'import' $(tput bold)$(tput setaf 7)$workspace_dir$json_file$(tput sgr0) 'into the fontello website to preserve already used fonts.'
        echo
        exit 1
    else
	echo
	echo '* UNZIPPING *'
	echo 'Extracting Archive:' $(tput bold)$(tput setab 1)$get_latest_fontello_zip_file$(tput sgr0) '...'
	fi
}

function unzip_fontello_zip_file {
	#continu unzipping fontello-NUMBER/CHARS.zip file to Downloads/fontello/ directory
    echo
    "${unzip_fontello_zip_file_to_fontello_dir[@]}"
    echo
    echo '...to:' $json_dir
    #exit 1
}

function create_local_usr_share_font_dir {
	if [ ! -d "$local_dir" ]; then
	# if DIRECTORY doesn't exist.
	echo
	echo 'Do you want to create the dir '
	echo	$local_dir '?'
	echo

	read -p '[Y/N] $ ' -n 1 -r
	echo

	# exit program when user doesn't want to create DIRECTORY
	if [[ ! $REPLY =~ ^[Yy]$ ]]
	then
		echo 'exit'
		echo
		exit 1
	else
	# User approved, continue creating directory /usr/share/fonts/truetype/fontello
		echo
		command_create_font_dir=(sudo mkdir -p $local_dir)
		"${command_create_font_dir[@]}"

	fi
fi

if [ ! -d "$font_dir" ]; then
	# if DIRECTORY doesn't exist
	echo
	echo 'Do you want to create the dir '
	echo    $font_dir  '?'
	echo

	read -p '[Y/N] $ ' -n 1 -r
	echo

	# exit program when user doesn't want to create DIRECTORY
	if [[ ! $REPLY =~ ^[Yy]$ ]]
	then
	        echo 'exit'
	        echo
	        exit 1
	else
	# User approved continu creating directory $HOME/Downloads/fontello/font/
		command_create_download_font_dir=(mkdir -p $font_dir)
		"${command_create_download_font_dir[@]}"
	fi
fi
}

function edit_demo_and_css_file {
	"${edit_demo_html_file[@]}"

	"${edit_fontello_css_file[@]}"
}

# Create fontello directory if it doesn't exist in Downloads
create_if_fontello_dir_doesnt_exists

# Check if fontello-NUMBER/CHARS.zip exists and asign to variable
check_if_fontello_zip_file_exists

# Unzip fontfile to fontello directory
unzip_fontello_zip_file

# create the font dir in /usr/share/fonts/truetype/fontello/ if it doesn't exist
create_local_usr_share_font_dir

echo -e '\t' "${green_done[@]}"

echo
echo '* ONE MOMENT PLEASE ... '
echo '* copying fonts from Downloads to local and project directories'
echo

# for loop to send everything to the 1st & 2nd ring of Dantes inferno
for i in $font_dir$font_filename
do
	echo '* COPY *'
	echo ''
	echo ' ' $i ' > ' $local_dir

	"${command_cp_font_to_lcl_dir[@]}"

	echo -e '\t' "${green_done[@]}"
	echo ' ' $i ' > ' $workspace_dir
	echo -e '\t' "${green_done[@]}"

	"${command_cp_font_to_workspace[@]}"
	echo

	echo '* FLUSH *'
	echo
	echo ' reloading ' $local_dir ' to add the fonts to the local system'

	"${command_flush_cache[@]}"

	echo -e '\t' "${green_done[@]}"
	echo

	echo '* COPY *'
	echo
	echo '* create in dir: ' $font_dir
	"${listing_of_local_dir[@]}"
	echo -e '\t' "${green_done[@]}"
	echo

	echo '* create in dir: ' $workspace_dir
	"${listing_of_workspace_dir[@]}"
	echo -e '\t' "${green_done[@]}"
	echo

	#exec $json_file copy
	"${command_cp_json_file_tp_workspace[@]}"
	"${listing_of_json_file[@]}"
	echo -e '\t' "${green_done[@]}"
	echo

	echo $json_dir$json_file ' > ' $workspace_dir$json_file
	echo -e '\t' "${green_done[@]}"

	echo
	echo
	echo -e '\t' "${all_done[@]}"
	echo $'\n'
done

# sed "css" string instances from demo.html and *.css files file because of flattening dir structure of zipfile
edit_demo_and_css_file

exit 0
