#!/usr/bin/env python

import re
import os
import sys
import logging

from optparse import OptionParser
from twisted.python.reflect import namedAny
from django.conf import settings
import sqlalchemy
from sqlalchemy.orm import scoped_session
from sqlalchemy.orm.session import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from urlparse import urlparse

logging.basicConfig(
    level = logging.DEBUG,
    format = '%(asctime)s - %(levelname)s - %(message)s',
)
logger = logging.getLogger()



def sys_exec(cmd):
    logger.info("executing: %s" % cmd)
    if os.system(cmd) != 0:
        raise Exception('Command returned with failure code')


def make_session(conn_str):
    #db_engine = sqlalchemy.create_engine(conn_str, echo=False)
    #session = scoped_session(sessionmaker(bind=db_engine, autoflush=True, autocommit=True))
    #conn = db_engine.connect()
    #metadata = sqlalchemy.MetaData(db_engine)

    engine = sqlalchemy.create_engine(conn_str, echo=False, convert_unicode=True)
    Session = sessionmaker(bind=engine)
    return Session(), engine

def get_tables(metadata, exclude_tables, include_tables):

    rv = []

    # loop source apps
    for app in settings.INSTALLED_APPS:

        try:
            sa_datamapper = namedAny(app+'.sa_datamapper')
        except AttributeError, e:
            continue

        sa_datamapper.do_tables(metadata)
        for x in sa_datamapper.get_tables().values():
            table_name = unicode(x)

            for s in exclude_tables:
                if re.match(s, table_name):
                    logger.info('skipping table %s' % table_name)
                    break
            else:
                rv.append(table_name)

    rv += [x for x in include_tables if x not in rv]

    return rv

def get_dump(from_db, hostname, exclude_tables, include_tables):

    source, sengine = make_session(from_db)
    smeta = sqlalchemy.MetaData(bind=sengine)
    tables = get_tables(smeta, exclude_tables, include_tables)

    from_url = urlparse(from_db)

    assert from_url.scheme == 'mysql'

    # takes 'iqportal2' from '/iqportal2?charset=utf8'
    from_db = re.findall('\/([^?]+)', from_url.path)[0]

    from_ssh_host = from_url.hostname

    dump_cmd = """mysqldump --opt --skip-routines --skip-lock-tables --no-create-db -u "%(from_user)s" -p"%(from_pass)s" -h "%(from_host)s" -P "%(from_port)s" "%(from_db)s" %(from_tables)s""" % {
        'from_user': from_url.username,
        'from_pass': from_url.password,
        'from_host': from_url.hostname,
        'from_port': from_url.port,
        'from_db': from_db,
        'from_tables': ' '.join(tables),
    }

    ssh_dump_cmd = """ssh %(ssh_host)s "%(dump_cmd)s | gzip > /tmp/cpdb.gz; exit \${PIPESTATUS[0]}" """ % {
        'ssh_host': from_ssh_host,
        'dump_cmd': dump_cmd.replace('"', '\\"'),
    }

    logger.info("Dumping database...")
    sys_exec(ssh_dump_cmd)

    logger.info("Copying database..")
    ssh_cp_cmd = """scp %(from_ssh_host)s:/tmp/cpdb.gz %(to_ssh_host)s:/tmp/cpdb.gz""" % {
        'from_ssh_host': from_ssh_host,
        'to_ssh_host': hostname,
    }

    sys_exec(ssh_cp_cmd)
    return '/tmp/cpdb.gz', from_ssh_host


def cpdb(hostname_gz, import_gz, to_db, do_drop):

    assert import_gz is not None

    to_url = urlparse(to_db)
    assert to_url.scheme == 'mysql'
    assert to_db != 'iqportal'

    to_db = re.findall('\/([^?]+)', to_url.path)[0]

    mysql_cmd = """
        mysql --max_allowed_packet=1024M -u "%(to_user)s" -p"%(to_pass)s" -h "%(to_host)s" -P "%(to_port)s" "%(to_db)s"
    """ % {
        'to_user': to_url.username,
        'to_pass': to_url.password,
        'to_host': to_url.hostname,
        'to_port': to_url.port,
        'to_db': to_db,
    }

    # drop/create db
    if do_drop:
        recreate_cmd = """
            echo 'SET FOREIGN_KEY_CHECKS=0; DROP DATABASE %(to_db)s; CREATE DATABASE %(to_db)s' | %(mysql_cmd)s
        """ % {
            'to_db': to_db,
            'mysql_cmd': mysql_cmd
        }
        logger.info("Re-creating database %s" % (to_db))

        ssh_cmd = """ssh %(ssh_host)s "%(recreate_cmd)s" """ % {
            'ssh_host': hostname_gz,
            'recreate_cmd': recreate_cmd.replace('"', '\\"'),
        }

        sys_exec(ssh_cmd)

    # importing
    import_cmd = """test -f %(import_gz)s && zcat %(import_gz)s | %(sed)s | %(mysql_cmd)s""" % {
        'import_gz': import_gz,
        'sed': 'sed -e "s/[a-zA-Z0-9._-]\+@[a-zA-Z0-9._-]\+/cpdb@mail-test.basesoft.com/g"',
        'mysql_cmd': mysql_cmd
    }
    logger.info("Importing database from file %s on host %s to db %s" % (import_gz, hostname_gz, to_db))

    ssh_import_cmd = """ssh %(ssh_host)s "%(import_cmd)s" """ % {
        'ssh_host': hostname_gz,
        'import_cmd': import_cmd.replace('"', '\\"'),
    }

    sys_exec(ssh_import_cmd)

def process_apps(db_conn_str):

    engine = sqlalchemy.create_engine(db_conn_str, echo=False, convert_unicode=True)
    conn = engine.connect()

    for app in settings.INSTALLED_APPS:
        try:
            cp_db_module = namedAny(app+'.cpdb')
        except AttributeError:
            continue
        logger.info('calling cbdb.post_process on app %s' % app)
        try:
            cp_db_module.post_process(conn)
        except Exception, e:
            logger.warning("Failed to execute post_process on app %s: %s" % (app, unicode(e)))

if __name__ == '__main__':

    parser = OptionParser()
    parser.add_option("--source", dest="source", help="source settings module or file (.gz)")
    parser.add_option("--exclude-tables", dest="exclude_tables", help="comma seperated list of tables to skip when dumping (regexes allowed)")
    parser.add_option("--include-tables", dest="include_tables", help="comma seperated list of tables to add (regexes allowed)")
    parser.add_option("--include-agg", action="store_true", default=False, dest="include_agg", help="if set, include 'agg.*' tables in dump")
    parser.add_option("--no-drop", action="store_true", default=False, dest="no_drop", help="if set, dont re-create the database")

    (options, args) = parser.parse_args()
    logger.info('options: %s' % options)

    # exclude tables
    exclude_tables = []
    if options.exclude_tables:
        exclude_tables += options.exclude_tables.split(',')
    if not options.include_agg:
        exclude_tables.append('agg.*')

    include_tables = []
    if options.include_tables:
        include_tables += options.include_tables.split(',')

    # re-creates database if set (default true)
    do_drop = not options.no_drop

    if not options.source:
        print 'missing source'
        sys.exit(1)

    # check valid target module
    if os.environ['DJANGO_SETTINGS_MODULE'].endswith('stable'):
        raise Exception('i refuse to copy to stable')

    target_module = namedAny(os.environ['DJANGO_SETTINGS_MODULE'])
    logger.info('target: %s' % target_module)

    to_url = urlparse(target_module.DB_CONNSTR)
    target_hostname = to_url.hostname

    import_gz = None

    cmd_delete_src = None
    cmd_delete_target = None

    # check if the source is a pointer to a filename
    if options.source.endswith('.gz'):

        # check if the filename is a remote file. then copy it to the target
        source = options.source
        if len(source.split(':')) == 2:
            scp_cmd = """
            scp %(source)s %(target_hostname)s:/tmp/cpdb.gz
            """ % {
                'source': source,
                'target_hostname': target_hostname
            }

            sys_exec(scp_cmd)

            import_gz = '/tmp/cpdb.gz'
            cmd_delete_target = 'ssh %s "[ -f %s ] && rm %s"' % (target_hostname, import_gz, import_gz)

        # else it must be a local file
        else:
            source = os.path.expanduser(options.source)
            assert os.path.exists(source)
            import_gz = source
            target_hostname = '127.0.0.1'

    # else, the source is a django settings module
    else:
        try:
            source_module = namedAny(options.source)
        except Exception, e:
            raise Exception('invalid module: %s: %s' % (options.source, unicode(e)) )
        assert source_module != target_module
        logger.info('source: %s' % source_module)

        import_gz, from_ssh_host = get_dump(source_module.DB_CONNSTR_READONLY, target_hostname, exclude_tables, include_tables)

        cmd_delete_src = 'ssh %s "[ -f %s ] && rm %s"' % (from_ssh_host, import_gz, import_gz)
        cmd_delete_target = 'ssh %s "[ -f %s ] && rm %s"' % (target_hostname, import_gz, import_gz)


    cpdb(target_hostname, import_gz, target_module.DB_CONNSTR, do_drop)

    process_apps(target_module.DB_CONNSTR)

    # cleanup
    if cmd_delete_src is not None:
        sys_exec(cmd_delete_src)

    if cmd_delete_target is not None:
        sys_exec(cmd_delete_target)

    print 'done.'
