#!/usr/bin/env python

import sqlalchemy
from sqlalchemy.orm import scoped_session, sessionmaker
from twisted.python.reflect import namedAny

from django.conf import settings
from basesoft.sqlalchemy.sqlalchemy_schemadisplay import create_schema_graph

db_engine = sqlalchemy.create_engine(settings.DB_CONNSTR, echo=False)
session = scoped_session(sessionmaker(bind=db_engine, autoflush=True, autocommit=True))
conn = db_engine.connect()
metadata = sqlalchemy.MetaData(db_engine)

for app in settings.INSTALLED_APPS:
    try:
        sa_module = namedAny(app+'.sa_datamapper')
    except AttributeError, e:
        continue

    if hasattr(sa_module, 'reset'):
        sa_module.reset()

    if hasattr(sa_module, 'do_tables'):
        sa_module.do_tables(metadata)


# create the pydot graph object by autoloading all tables via a bound metadata object
graph = create_schema_graph(
    metadata=metadata,
    show_datatypes=False, # The image would get nasty big if we'd show the datatypes
    show_indexes=False, # ditto for indexes
    #rankdir='LR', # From left to right (instead of top to bottom)
    concentrate=False # Don't try to join the relation lines together
)

graph.write_png('dbschema.png') # write out the file
