#!/bin/bash

#************************************************
# INSTRUCTIONS:
# Run script as user, e.g:
# ./setenv_iqportal
#************************************************

# Commands to execute
git_pull_master=(git pull master)

# vars
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
DIR_related_bin="$DIR/"

runserver="runserver.sh"
syncdb="syncdb.sh"
resetdb="resetdb.sh"

# green|all DONE sign
green_done=($(tput bold)$(tput setaf 2)DONE$(tput sgr0))

#echo $DIR

# function
show_menu() {
	echo
	echo 'What would you like to do?'
	echo
	echo ' 1. start Django server [./related/bin/runserver.sh]'
	echo ' 2. sync database [./related/bin/syncdb.sh] and runserver [./related/bin/runserver.sh]'
	echo ' 3. reset database [./related/bin/resetdb.sh]'
	echo ' 4. kill all running Django process?'
	#echo ' 5. git pull master (not available yet)'
	#echo ' 6. git rebase master (need to be in branch! | not available yet)'
	echo
}

# function
run_command() {
	run=$($DIR_related_bin$1)

	echo
	echo ' Running          -> $' $run
	echo

	"${$run[@]}"
}

# function
run_source() {
	dir_rb=$DIR_related_bin
	vars="vars"

	a=${dir_rb}$vars
	run_source_command=$(source $a)

	echo
	echo ' Running          -> $ source '$a
	echo

}

#function
run_django_server() {
	echo
	echo ' Set Python $PATH ->' $PYTHON_BIN
	echo ' Running          ->./'${DIR_related_bin}manage.py runserver 0.0.0.0:8000 --settings=basesoft.settings.$USER
	echo
	echo -e '\t' "${green_done[@]}"
	echo

	if [ "$PYTHON_BIN" == "" ]; then
	        PYTHON_BIN="python"
	fi

	export PYTHONWARNINGS="d"
	$PYTHON_BIN $DIR_related_bin/manage.py \
	runserver 0.0.0.0:8000 --settings=basesoft.settings.$USER

}

#function
run_syncdb() {

	dir_rb=$DIR_related_bin
	dot="."

	a=$dot${dir_rb}$1
	run_source_command=$(./$a)

	echo
	echo ' Running          -> $ ./'$a
	echo

}

#function
run_resetdb() {

	dir_rb=$DIR_related_bin
	dot="."

	a=$dot${dir_rb}$1
	run_source_command=$($a)

	echo
	echo ' Running          -> $ ' $a
	echo

}

#function
check_django() {
	#run=$(ps aux | grep -i runserver)

	run=$(ps -ef | grep "runserver" | awk '{print $2}' | xargs kill)

        echo
        echo ' Running          -> $' $run
        echo

}

#Prompt the menu to the user
show_menu

read -p 'Type a [number] to run  or [N] to exit program $ ' -n 1 -r
echo

if [[ ! $REPLY =~ ^[0-9]$ ]]; then

	echo 'EXIT'
	echo
	exit 0

elif [[ $REPLY == 1 ]]; then

	echo
	echo ' Start Django server'
	echo
	run_source $DIR_related_bin

	echo
	echo -e '\t' "${green_done[@]}"

	run_django_server

	echo
	exit 0

elif [[ $REPLY == 2 ]]; then

	echo
	echo ' Sync database and Runserver'
	echo
	run_source $DIR_related_bin

	echo
	echo -e '\t' "${green_done[@]}"

	run_syncdb $syncdb

	echo
	echo -e '\t' "${green_done[@]}"

	run_django_server

	echo
	exit 0

elif [[ $REPLY == 3 ]]; then

	echo
	echo ' Reset local database'
	echo
	run_source $DIR_related_bin

	echo
	echo -e '\t' "${green_done[@]}"

	run_resetdb $resetdb

	echo
	echo -e '\t' "${green_done[@]}"

	echo
	exit 0


elif [[ $REPLY == 4 ]]; then

	echo
	echo ' Check PID of Django '
	echo
	check_django

	echo
	exit 0

elif [[ $REPLY == 5 ]]; then

	echo '5'
	echo
	exit 0

else
	echo ' Not in menu - EXIT program'
	echo
	exit 0
fi

exit 0
