#!/usr/bin/python

# tests all installed iqportal apps

from django.conf import settings
from twisted.python.reflect import namedAny

import os.path

import getopt, sys
import subprocess
import commands

orig_stdout = sys.stdout
orig_stderr = sys.stderr

#sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)


def run_tests(testtype):

    assert testtype in ('unit', 'selenium')

    failures = []
    subunit_outputs = []

    trial_apps = settings.TRIAL_APPS
    if trial_apps is None:
        trial_apps = settings.INSTALLED_APPS

    #print 'running tests for apps: %s' % unicode(trial_apps)

    for app in trial_apps:

        if app.endswith('.test'):

            try:
                module = namedAny(app+'.'+testtype)
            except AttributeError:
                continue

            test_module = '%s.%s' % (app, testtype)

            #print "\n*************************\n** %s\n*************************\n\n" % test_module

            args = [
                '/usr/bin/trial',
                #'-n',
                '--reporter=subunit',
                test_module
            ]

            filename = '%s.subunit' % app
            subunit_outputs.append(filename)

            #sys.stdout = open(filename, 'w')
            sys.stderr = open(filename, 'w')

            f = open(filename, 'w')

            p = subprocess.Popen(args, stdout=subprocess.PIPE)
            for line in p.stdout:
                sys.stdout.write(line)
                f.write(line)

            f.close()

            status = os.waitpid(p.pid, 0)[1]

            if status != 0:
                failures.append(test_module)

            # restore stdout/err and print test output
            sys.stdout = sys.__stdout__
            sys.stderr = sys.__stderr__

            #print open(filename).read()
            #print
            #print

    # send results to trial_report
    for subunit_output in subunit_outputs:
        print 'sending: %s' % subunit_output
        #trial_report_submit_file(open(subunit_output).read())


    print """

        ****************************************
        * failures details:
        ****************************************

    """

    assert len(subunit_outputs) > 0

    p1 = subprocess.Popen(["cat"] + subunit_outputs, stdout=subprocess.PIPE)
    p2 = subprocess.Popen(["subunit-filter", "--no-skip", "--no-success"], stdin=p1.stdout, stdout=subprocess.PIPE)
    p1.stdout.close()
    output = p2.communicate()[0]
    print output


    print """

        ****************************************
        * failures:
        ****************************************

    """

    p1 = subprocess.Popen(["cat"] + subunit_outputs, stdout=subprocess.PIPE)
    p2 = subprocess.Popen(["subunit-filter", "--no-skip", "--no-success"], stdin=p1.stdout, stdout=subprocess.PIPE)
    p3 = subprocess.Popen(["egrep", "^failure: |^error: "], stdin=p2.stdout, stdout=subprocess.PIPE)
    p1.stdout.close()
    p2.stdout.close()
    output = p3.communicate()[0]
    print output

    print """

        ****************************************
        * failures stats:
        ****************************************

    """

    p1 = subprocess.Popen(["cat"] + subunit_outputs, stdout=subprocess.PIPE)
    p2 = subprocess.Popen(["subunit-stats"], stdin=p1.stdout, stdout=subprocess.PIPE)
    p1.stdout.close()
    output = p2.communicate()[0]
    print output

    sys.exit(int(len(failures)))

import httplib, urllib

def trial_report_submit_file(file_contents):

    if settings.TRIAL_REPORT_URL is None:
        print 'Not sending report: No TRIAL_REPORT_URL configured'
        return

    params = urllib.urlencode({
        'project': settings.TRIAL_PROJECT,
        'buildnumber': os.environ['BUILDBOT_BUILDNUMBER'],
        'buildername': os.environ['BUILDBOT_BUILDERNAME'],
        'contents': file_contents
    })

    headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}

    conn = httplib.HTTPConnection(settings.TRIAL_REPORT_URL[0])
    conn.request("POST", settings.TRIAL_REPORT_URL[1], params, headers)
    response = conn.getresponse()
    #print response.reason
    data = response.read()
    if response.status != 200:
        print
        print 'ERROR sending to trial_report:'
        print data
        print
        print

    print 'data: %s' % data
    conn.close()

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "t:", ["testtype="])
    except getopt.GetoptError, err:
        print str(err)
        sys.exit(2)

    testtype = None

    for o, a in opts:
        if o in ("-t", "--testtype"):
            testtype = a
        else:
            assert False, "unhandled option"


    run_tests(testtype)




if __name__ == "__main__":
    main()


