'''
Created on Aug 27, 2014

@author: marten
'''

from django.utils import simplejson
from django.conf import settings

from basesoft.web.core.util.db import DatabaseSession
from iqportal.iqube.core.models import DataSourceCalculatedMember, MdxMethod, MdxDataSourceCalculatedMember,\
    MdxCalculatedMember, DataSource
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder

from iqportal.iqube.core.controllers import MondrianMemberSerialize
from basesoft.web.core.util.controllers import AbstractController

sa_session = DatabaseSession().Session

def convert():
    """
    Converts the old DataSourceCalculatedMember to the new MdxDatasourceCalculatedMember
    """
    _map = {}

    # all current calculated members are aggregates
    meth = sa_session.query(MdxMethod).filter(MdxMethod.name=='Aggregate').one()
    # all current calculated members are part of the dimension Attribute
    dimension = 'Position'

    ds = sa_session.query(DataSource).filter(DataSource.name=='%s.TNT-CLM' % settings.SETTINGS_MODULE).first()
    rl = ds.role_level
    ds.role_level = 0
    client = ds.connect(sa_session)

    ac = AbstractController(sa_session)

    old = sa_session.query(DataSourceCalculatedMember).filter(DataSourceCalculatedMember.deleted==False).all()
    for old_cm in old:

        items = old_cm.formula.split('+')
        selection = []
        for m in items:
            member = client.get_member_tree_op(old_cm.cube, m, client.MDTREEOP_SELF, 'MEMBER_UNIQUE_NAME')[0]
            selection.append(MondrianMemberSerialize(member))

        formula = {
            "inputs": [{
                "inputs": [],
                "selection": selection,
                "objectName": "MdxDataSourceExpressionItem",
                "posXY": [512, 190],
                "loaderName": "iqube_datasource_expression_item",
                "return_type": "set",
                "type": "set",
                "dimension": "[Pool]"
            }],
            "method_id": meth.id,
            "objectName": "MdxMethodItem",
            "loaderName": "iqube_mdx_method_item",
            "return_type": meth.return_type,
            "posXY": [512, 485]
        }

        cm = MdxDataSourceCalculatedMember()
        cm.datasource = old_cm.datasource
        cm.cube = old_cm.cube
        cm.label = old_cm.caption
        cm.type = MdxCalculatedMember.CALCULATED_MEMBER
        cm.input_count = 0
        cm.dimension = dimension
        cm.formula = simplejson.dumps(formula, ensure_ascii=False, cls=JsonRpcJSONEncoder)
        ac.grant_privilege_to(cm, privileges=cm.all_privileges, group=True, target_group=old_cm.groups[0].group, check_own_privs=False)
        ac.grant_privilege_to(cm, privileges=cm.PRIV_VIEW, group=True, license=True, target_group=old_cm.groups[0].group, check_own_privs=False)
        sa_session.add(cm)
        sa_session.flush()
        _map[old_cm.id] = cm.id

    # place the original role level back
    ds.role_level = rl
    sa_session.add(ds)
    sa_session.flush()

    update_sql = """
        UPDATE `tblTntCLMDashboardWidgetFilter` SET `calculated_member_id` = %(new_id)s WHERE `id` IN (%(ids)s)
    """

    query_sql = """
        SELECT `id`, `calculated_member_id` FROM `tblTntCLMDashboardWidgetFilter` WHERE NOT ISNULL(`calculated_member_id`)
    """
    original = {}
    for _id, cm_id in sa_session.execute(query_sql).fetchall():
        if not original.has_key(cm_id):
            original[cm_id] = []
        original[cm_id].append(_id)

    for old_id, new_id in _map.iteritems():
        sa_session.execute(update_sql % {
            'new_id': new_id,
            'ids': ','.join([str(i) for i in original[old_id]])
        })

    sa_session.execute("""
        ALTER TABLE `tblTntCLMDashboardWidgetFilter` CHANGE COLUMN `calculated_member_id` `calculated_member_id` INT(11) NULL DEFAULT NULL  ,
          ADD CONSTRAINT `tblTntCLMDashboardWidgetFilter_ibfk_3`
          FOREIGN KEY (`calculated_member_id` )
          REFERENCES `tblMdxCalculatedMember` (`id` )
          ON DELETE CASCADE
    """)

    print 'NOTE: it is now save to remove TntCLMBusinessFunction, DataSourceCalculatedMemberGroup & DataSourceCalculatedMember from the datamappers'

if __name__ == "__main__":
    sa_session.begin()
    try:
        convert()
        sa_session.commit()
    except Exception, e:
        sa_session.rollback()
        raise e
