# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
buildbot settings
"""

import os

from basesoft.settings.debug import *
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities

# set buildbot umask so that group and world can read all files created
# this is here cuz buildbot slave has default umask of 0077 (not readable)
os.umask(0022)

SELENIUM['timeout'] = 240 * 1000
SELENIUM['closeBrowser'] = True

EMAIL_HOST = 'basesoft-fs.office.basesoft.com'
SERVER_EMAIL= 'buildbot@testorix.dev.ams'

DB_CONNSTR_FOODMART = 'mysql://foodmart:foodmart@testorix.dev.ams:3306/foodmart'

# override debug logging so we only see errors in stdout
LOGGER_STDOUT_LEVEL = logging.ERROR
LOGGER_SYSLOG_LEVEL = logging.DEBUG

MONDRIAN_XMLA_SERVER = ('testorix.dev.ams', 8080)
MONDRIAN_XMLA_CACHE_ENABLED = True

TRIAL_PROJECT = 'iqportal'
TRIAL_REPORT_URL = ('trial-report.office.basesoft.com:80', '/buildbot/submit/')


# fixme: #3858
FONTS_PATH = None

# we dont want emails from buildbot
EMAIL_HOST = None

# send to vm.office selenium hub
SELENIUM['host'] = 'vm.office'
SELENIUM['port'] = '4444'


def setup_buildbot(vars):

    base_url = 'http://testorix.dev.ams.basesoft.com:%s/' % vars['BUILDBOT_DJANGO_PORT']
    base = vars['BUILDBOT_BASE']
    writable = os.path.join(base, 'iqportal', 'writable')
    temp = os.path.join(writable, 'temp')

    iqube_instance = 'iqube-'+vars['BUILDBOT_INSTANCE']

    arvato_local_path = os.path.join(temp, 'arvato')

    vars.update({

        'BASE_URL': base_url,

        'WRITABLE_DIRECTORY': writable,
        'TMP_FILE_REPOSITORY':  temp,
        'IQREPORT_GRAPH_LOCATION': writable,
        'FILE_REPOSITORY': writable,
        'CACHE_DIR': writable,

        'VIRTUAL_DATE': os.path.join(writable, 'virtual_date.txt'),
        'SELENIUM_PYTHONPATH': vars['BUILDBOT_BASE'] + '/iqportal:' + vars['BUILDBOT_BASE'] + '/iqportal/related/libs',

        'SCHEDULER_SERVER_PORT':  vars['BUILDBOT_SCHEDULER_PORT'],
        'SCHEDULER_SERVER_XMLRPC': 'http://testorix.dev.ams:%s/' % vars['BUILDBOT_SCHEDULER_PORT'],
        'SELENIUM_DJANGO_ADDR': '0.0.0.0:%s' % vars['BUILDBOT_DJANGO_PORT'],

        'MONDRIAN_INSTANCE': iqube_instance,
        'MONDRIAN_XMLA_PATH': '/%s/xmla' % iqube_instance,

        'IQUBE_SERVLET': "http://testorix.dev.ams:8080/%s/IQubeXmlRpcServlet" % iqube_instance,

        'ARVATO_LOCAL_PATH': arvato_local_path,
        'ARVATO_GET_XML_CMD': 'mv %s/*.xml %s' % (os.path.join(temp, 'arvato_ftp'), arvato_local_path),

        'ORDER_OGONE_POST': "%sbasesoft.web.test/selenium/order/ogone_payment_accept/" % base_url,


    })

    vars['SELENIUM']['screenshot_path'] = '/var/lib/buildbot/public_html/screenshots/%s' % vars['BUILDBOT_INSTANCE']
    vars['SELENIUM']['desired_capabilities'] = DesiredCapabilities.CHROME
    vars['SELENIUM']['desired_capabilities']['platform'] = 'LINUX'
    vars['SELENIUM']['browserURL']  = 'http://testorix.dev.ams.basesoft.com:%s/' % vars['BUILDBOT_DJANGO_PORT']



    def _mkdir(path):
        if not os.path.exists(path):
            os.makedirs(path)
        assert os.access(path, os.W_OK)

    _mkdir(writable)
    _mkdir(temp)
    _mkdir(vars['STATIC_SQUASHED_ROOT'])

    # debug log to file
    from logging import FileHandler
    logger = logging.getLogger()
    logger.addHandler(FileHandler(os.path.join(writable, 'application.log')))


    return vars

