# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
URLconfs are how Django associates a given URL with given Python code.

TODO: Decouple all urls! We dont wanna point directly to a view here.
See: http://www.djangoproject.com/documentation/tutorial03/#decoupling-the-urlconfs
"""

from django.conf.urls.defaults import *
from django.conf import settings
import os
import sys


# standard
_urls = [
    (r'^$',include('basesoft.web.contrib.home.urls')),
    (r'^i18n/', include('django.conf.urls.i18n')),
    (r'^jsi18n/(?P<packages>\S+?)/$', 'django.views.i18n.javascript_catalog'),
]

# rosetta translation app
if 'basesoft.web.contrib.rosetta' in settings.INSTALLED_APPS:
    _urls += [
        url(r'^rosetta/', include('basesoft.web.contrib.rosetta.urls')),
    ]

# apps can provide urls too
for app in settings.INSTALLED_APPS:

    # escape regex chars
    regex_safe_app = app.replace('.', '\.')

    # app urls module
    try:
        _urls.append((r'^%s/' % regex_safe_app, include('%s.urls' % app)))
    except ImportError, e:

        if (e.message != "No module named urls"):
            print "Failed to import urls from %s: " % app
            print e

    # This is a handy trick to make sure the static-serving view doesn't slip into a production setting by mistake.
    # When going live we need to turn these into apache-Location directives
    if settings.SERVE_STATIC_MEDIA:
        module = __import__(app, {}, {}, [''])
        media_path = module.__path__[0] + '/media'
        if os.path.exists(media_path):
            _urls.append(
                (r'^site_media/%s/(?P<path>.*)$' % regex_safe_app, 'django.views.static.serve', {
                    'document_root': media_path,
                    'show_indexes': True
                })
            )

# static squashed media
if settings.SERVE_STATIC_MEDIA:
    _urls.append(
        (r'^static/(?P<path>.*)$', 'django.views.static.serve', {
            'document_root': settings.STATIC_SQUASHED_ROOT,
            'show_indexes': True
        })
    )



urlpatterns = patterns('',
    *_urls
)

