# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

"""
General purpose functions that can be used for string processing.
"""

import re

def trim_multiline(str, trim_empty_lines=False):
    """
    Remove the spaces from a multiple line string. Usually for strings defined with triple quotes that have useless
    spaces (because of code indentation).

    @param String str: the string that has multiple lines (works on simple strings too)
    @param Boolean trim_empty_lines: if true and the first and/or last lines are empty, they are removed.

    @return: a new string that doesn't contain spaces before or after each line.
    """

    stripped = re.compile('^\t+|^ +| +$| \t+$', re.MULTILINE).sub('', str)

    if trim_empty_lines:
        if stripped[0] == '\n': stripped = stripped[1:]    # remove the first line if empty
        if stripped[-1] == '\n': stripped = stripped[:-1]  # remove the last line if empty

    return stripped
