# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings
from sqlalchemy.orm.exc import NoResultFound

def sync(session):
    """
    Synchronizes all account configuration options
    """
    app_configs = []
    # go through the apps and see if they have permissions to sync.
    for app in settings.INSTALLED_APPS:
        try:
            sa_module = __import__(app + '.sa_datamapper', {}, {}, [''])
        except ImportError:
            # if we do not have a datamapper then just continue as there
            # are no permissions to be sync'ed
            continue

        if hasattr(sa_module, 'app_configs'):
            app_configs += sa_module.app_configs()

    ### sync Configs
    # Delete every config that is not defined in the above app_configs list.
    # All configs that are deleted are obviously out of date and no
    # longer used.

    from basesoft.web.contrib.account.models import Config

    existing_configs = session.query(Config).all()
    all_names = [c.name for c in app_configs]

    # delete configs that are not in apps anymore
    for existing_cf in existing_configs:
        if existing_cf.name not in all_names:
            session.delete(existing_cf)
    session.flush()

    # sync configs (add new or update)
    for config in app_configs:
        try:
            db_config = session.query(Config).filter(Config.name == config.name).one()
        except NoResultFound:
            db_config = Config()

        db_config.name = config.name
        db_config.description = config.description
        db_config.type = config.type
        db_config.default = config.default

        session.add(db_config)

    session.flush()
