# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.core.exceptions import SecurityException, LoginRequiredException
from django.utils.translation import ugettext as _
from django.http import HttpRequest

from basesoft import logging
from basesoft.web.core.util.controllers import PermissionController
logger = logging.getLogger('basesoft.account')

def login_required(method):
    """
    This validates whether or not a user is logged in. Can be used as decorator
    around views or any method that has a request argument
    """
    def is_logged_in(*args, **kwargs):
        for p in args:
            if isinstance(p, HttpRequest):
                if p.session.has_key('user_id'):
                    return method(*args, **kwargs)
                raise LoginRequiredException(_('Login required'))
            if isinstance(p, PermissionController):
                if p.get_account() is not None:
                    return method(*args, **kwargs)
                raise LoginRequiredException(_('Login required'))

        raise SecurityException('Method %s does not have a request argument.' % method)

    return is_logged_in

def assert_user_logged_in(method):
    """
    This decorator checks if the user is logged in, and if not, raises a
    SecurityException
    """
    def is_logged_in(*args, **kwargs):
        for arg in args:
            if isinstance(arg, HttpRequest):
                if not arg.session.has_key('user_id'):
                    raise LoginRequiredException(_('Login required'))
                return method(*args, **kwargs)
            if isinstance(arg, PermissionController):
                if arg.get_account() is None:
                    raise LoginRequiredException(_('Login required'))
                return method(*args, **kwargs)
    return is_logged_in

def assert_user_not_logged_in(method):
    """
    This decorator checks if the user is not logged in, and if it is, raises a
    SecurityException
    """
    def is_not_logged_in(*args, **kwargs):
        for arg in args:
            if isinstance(arg, HttpRequest):
                if arg.session.has_key('user_id'):
                    raise SecurityException(_('User is logged in'))
                return method(*args, **kwargs)
            if isinstance(arg, PermissionController):
                if arg.get_account() is not None:
                    raise SecurityException(_('User is logged in'))
                return method(*args, **kwargs)
    return is_not_logged_in
