# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.iconbar.models import IconBarIcon
from django.utils.translation import ugettext_lazy as _
from basesoft.web.contrib.account.decorators import assert_user_not_logged_in, assert_user_logged_in
from basesoft.web.core.exceptions import PermissionDeniedException

class RegistrationIcon(IconBarIcon):
    """
    Class responsible for handling the registration icon
    """

    title = _('Register')
    component = 'iq_registration_form'
    icon_index = 80
    group = 'register'

    char_main = u'\uE806' # Man icon
    char_sub = u'\uE81b' # Pencil icon

    @assert_user_not_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if when logged out
        """
        raise PermissionDeniedException('Disbled for the time being')

class IQPortalSettingsIcon(IconBarIcon):
    """
    Class responsible for handling the settings icon.
    No permission check is required as every account can open only its own settings panel.
    """

    id = 'icon-settings'
    title = _('Preferences')
    href = '#'
    icon_index = 70
    component = 'iq_account_preferences'
    group = 'identity'

    char_main = u'\uE80d' # Gears icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        The only requirement here is that user is logged in
        """
        pass

class AccountManagerIcon(IconBarIcon):
    """
    The account manager icon.
    """
    id = 'icon-account-manager'
    title = _('Accounts')
    href = '#'
    icon_index = 95
    component = 'iq_account_manager'
    group = 'administration'

    char_main = u'\uE823' # 4 lines icon
    char_sub = u'\uE806' # Man

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        """
        controller.assert_any_permissions('view_domain', 'view_license', 'view_all_domains')

def get_icons():
    """
    @param request: the request object that contains session, etc.
    """
    return [RegistrationIcon(), IQPortalSettingsIcon(), AccountManagerIcon()]
