# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings
from django.utils import translation
from sqlalchemy import and_
from basesoft.web.contrib.license.models import License

from sqlalchemy.orm.exc import NoResultFound
from basesoft.web.contrib.graph.models import GraphTheme
from basesoft.web.contrib.branding.models import Theme, LicenseTheme, Site

ALL_LICENSES = 1 # constant used for syncing themes for licenses




def _sync_themes(sa_session, themes):
    """
    create the themes if we don't have them
    """

    existing_themes = sa_session.query(Theme).all()
    existing_themes_modules = [e.module_name for e in existing_themes]

    existing_license_themes = sa_session.query(LicenseTheme).all()
    existing_license_themes_values = ['%s_%s' % (e.license.id, e.theme.id) for e in existing_license_themes]

    for t in themes:
        if t['module_name'] not in existing_themes_modules:
            nt = Theme()
            nt.name = t['name']
            nt.title = t['title']
            nt.module_name = t['module_name']
            nt.css_class = t['css_class']

            default_graph_theme = u'Default' # use the Default color set as default unless specified differently
            if t.has_key('default_graph_theme'):
                default_graph_theme = t['default_graph_theme']
            default_graph_theme = sa_session.query(GraphTheme).filter(GraphTheme.name==default_graph_theme).one()
            nt.default_graph_theme = default_graph_theme

            sa_session.add(nt)
        else:
            nt = sa_session.query(Theme).filter(Theme.module_name==t['module_name']).one()

        if t.has_key('licenses'):
            tl = t['licenses']
            if tl == ALL_LICENSES:
                licenses = sa_session.query(License).filter(License.self_branded==1).all()
            else:
                licenses = sa_session.query(License).filter(and_(License.self_branded==1, License.name.in_(tl))).all()

            for l in licenses:
                if '%s_%s' % (l.id, nt.id) not in existing_license_themes_values:
                    nlt = LicenseTheme()
                    nlt.license = l
                    nlt.theme = nt
                    if t.has_key('default') and t['default'] == l.name:
                        nlt.is_default = True

                    sa_session.add(nlt)
    sa_session.flush()

def post_sync(sa_session, app_name):
    """
    Sync objects only after app_name has ran it's sync()

    @param session: the sqlalchemy session
    @param app_name: the name of the app that has been sync-ed
    """
    if app_name == 'basesoft.web.contrib.graph':

        # sync themes only after color_sets have been created
        themes = [{
            'name': u'Icecream',
            'title': u'',
            'module_name': u'iq_theme_icecream',
            'css_class': u'icecream',
            'licenses': ['BaseSoft Solutions B.V.'],
            'default_graph_theme': u'Default',
        }, {
            'name': u'BaseSoft Solutions B.V.',
            'title': u'',
            'module_name': u'basesoft_theme_default',
            'css_class': u'basesoft',
            'licenses': ['BaseSoft Solutions B.V.'],
            'default': 'BaseSoft Solutions B.V.',
            'default_graph_theme': u'Default',
        }, {
            'name': u'Newcom - Research & Consultancy', # this should be defined in client app.
            'title': u'Newcom - Research & Consultancy',
            'module_name': u'newcom_theme_default',
            'css_class': u'newcom',
            'licenses': ['BaseSoft Solutions B.V.', 'Newcom - Research & Consultancy'],
            'default': 'Newcom - Research & Consultancy',
            'default_graph_theme': u'Default',
        }, {
            'name': u'YUI Sam',
            'title': u'',
            'module_name': u'iq_theme_debug',
            'css_class': u'debug',
            'licenses': ['BaseSoft Solutions B.V.'],
            'default_graph_theme': u'Default',
        }]
        _sync_themes(sa_session, themes)


        # make the default Site object after themes are synced.
        # this is created once so it can be changed if needed
        try:
            default_site = sa_session.query(Site).filter(Site.host_match=='').one()
        except NoResultFound:

            default_site = Site()
            default_site.host_match = ''
            default_site.order = 999
            default_site.theme = sa_session.query(Theme).filter(Theme.module_name=='basesoft_theme_default').one()
            default_site.home_app = 'website'
            default_site.trac_conn_str = 'http://'

            sa_session.add(default_site)
            sa_session.flush()



