# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.account.decorators import login_required
from basesoft.web.contrib.branding.controllers import HeaderFooterController, LicenseThemeController,\
    SiteController
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.input.validation import plugins

class HeaderFooterRPCHandler:
    @login_required
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.controller = HeaderFooterController(request.Session(), account_id=request.session['user_id'])

    def load_state(self, id):
        """
        Load the requested header/footer.
        @param id: {int} the id of the header/footer we want to load
        @return: the state of the given header/footer
        """
        assert isinstance(id, int)
        state = self.controller.load_state(id)
        return state

    def save_state(self, state):
        """
        Save the header/footer from the state and return the id
        @param state: {dict} the state of the header/footer we want to save. if it
            has a dbID set, it will update, if it's not present, save new
        @return: the id of the saved header/footer
        """
        assert isinstance(state, dict)
        hf = self.controller.save_state(state)
        return hf.id

    def get_header_footers(self):
        hf = self.controller.get_header_footers()
        return [{'label': e.name, 'value': e.id } for e in hf]

    def get_header_footers_content(self, value):
        hf = self.controller.get_header_footers(int(value))
        if len(hf) == 0:
            # no header/footer found with that id
            return ''

        return hf[0].content

class SiteHandler:
    def __init__(self, request, **kwargs):
        self._sa_sesion = request.Session()
        self._controller = SiteController(self._sa_sesion, account_id=request.session['user_id'], request=request)

    def get_themes(self):
        c = LicenseThemeController(self._sa_sesion, account_id=self._controller.get_account().id, request=self._controller.request)
        return [{'label': theme.title if theme.title != '' else theme.name, 'value': theme.id} for theme in c.get_available_themes()]

    def get_all_themes(self):
        c = LicenseThemeController(self._sa_sesion, account_id=self._controller.get_account().id, request=self._controller.request)
        return [{'label': theme.title if theme.title != '' else theme.name, 'value': theme.id} for theme in c.get_all_themes()]

    def save_themes_for_licenses(self, themes, licenses):
        c = LicenseThemeController(self._sa_sesion, account_id=self._controller.get_account().id, request=self._controller.request)
        c.save_themes_for_licenses(themes, licenses)
        return True

    def delete_license_theme(self, license_id, theme_id):
        c = LicenseThemeController(self._sa_sesion, account_id=self._controller.get_account().id, request=self._controller.request)
        c.delete_license_theme(license_id, theme_id)
        return True

    def delete_site(self, ids):
        return self._controller.delete_site(ids)

    def load_site(self, host_name):
        return self._controller.load_site_state(host_name)

    def save_site(self, values):
        self._controller.save_or_update_site(**values)
        return True

    def validate_site(self, args):
        form_handler = FormValidator({
            'order': [(plugins.Required, {}), (plugins.Integer, {'unsigned': True, 'allow_zero': False}), ],
            'theme_id': [(plugins.Required, {})],
            'home_app': [(plugins.Required, {})],
            'analytics_id': [(plugins.Required, {})],
        })
        return form_handler.validate(args)

handlers = (HeaderFooterRPCHandler, SiteHandler, )
