# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class IQPortalContactIcon(IconBarIcon):
    """
    Class responsible for handling the contacts icon
    """

    id = 'icon-contact-manager'
    title = _('Contacts')
    href = '#'
    icon_index = 60
    component = 'iq_contact_manager'
    group = 'contacts'

    char_main = u'\uE807' # Users icon
    #char_sub = u'' # None

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('view_contact_list','view_contact')

class IQPortalImportContactIcon(IconBarIcon):
    """
    Class responsible for handling the contacts icon
    """

    id = 'icon-import-contacts'
    title = _('Import contacts')
    href = '#'
    icon_index = 62
    component = 'iq_contact_import_wizard'
    group = 'contacts'

    char_main = u'\uE807' # Users icon
    char_sub = u'\uE818' # Arrow left icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('import_contacts')

class IQPortalContactFieldMappingIcon(IconBarIcon):
    """
    Class responsible for handling the field mapping icon
    """

    id = 'icon-contact-field-mapping'
    title = _('Field mapping manager')
    href = '#'
    icon_index = 61
    component = 'contact_field_mapping'
    group = 'contacts'

    char_main = u'\uE807' # Users icon
    char_sub = u'\uE80d' # Gears icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('use_field_mapping_presets')

class IQPortalImportStatusIcon(IconBarIcon):
    id = 'icon-contact-import-status'
    title = _('Import status overview')
    href = '#'
    icon_index = 63
    component = 'iq_contact_file_import'
    group = 'contacts'

    char_main = u'\uE807' # Users icon
    char_sub = u'\uE818' # Arrow left icon

    @assert_user_logged_in

    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('import_contacts')

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [IQPortalContactIcon(), IQPortalImportContactIcon(), IQPortalContactFieldMappingIcon(), IQPortalImportStatusIcon(), ]
