# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel
from basesoft.web.contrib.dashboard.models import Dashboard

from sqlalchemy import and_, not_, or_, desc, func, Integer, String, Date, DateTime, Boolean, DECIMAL
from basesoft.web.contrib.dashboard.controllers import DashboardStateController
from basesoft.web.core.util.controllers import StateManager

class DashboardCustomQuery(CustomQuery):
    def tables(self):
        return (
            CustomQueryModel(Dashboard),
        )

    def fields(self):
        return (
            CustomQueryField(Dashboard.id, _('ID'), key="id"), # required for 'manage object privs'
            CustomQueryField(Dashboard.name, _('Name'), key="name", is_filter=True),
            CustomQueryField(Dashboard.id, _('Sharing'), key="sharing"),
            CustomQueryField(Dashboard.created, _('Created')),
            CustomQueryField(Dashboard.last_updated, _('Last updated')),
        )

    def _get_query_filter_on_privileges(self, *args, **kwargs):
        filter, fo = CustomQuery._get_query_filter_on_privileges(self, *args, **kwargs)
        dc = DashboardStateController(self.sa_session, StateManager(None, self.get_account().id, Dashboard.MODE_COMPOSE))
        filter = and_(filter, Dashboard.id.in_([d.id for d in dc.get_dashboards()]) )

        return filter, fo

custom_queries = (DashboardCustomQuery, )

