# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

"""
models for dashboard
"""

from sqlalchemy.orm.session import Session
from sqlalchemy.sql.expression import and_

from basesoft.web.core.models import ComposePrivilegeObject

class Dashboard(ComposePrivilegeObject):
    """
    @DynamicAttrs
    """
    _name_property = 'name'

    def _on_mark(self):
        session = Session.object_session(self)
        unmark = session.query(DashboardAccount).filter(and_(DashboardAccount.dashboard==self, DashboardAccount.is_default==True)).all()
        for ad in unmark:
            ad.is_default = False
            session.add(ad)

    on_mark_as_deleted = _on_mark
    on_mark_as_archived = _on_mark

    def __repr__(self):
        return '<Dashboard id=%s, name="%s">' % (self.id, self.name)

    def get_group_object(self):
        return (DashboardGroup, 'dashboard')

class DashboardAccount(object):
    """
    @DynamicAttrs
    """

class DashboardGroup(object):
    """
    @DynamicAttrs
    """

class DashboardTab(object):
    """
    Tabs in the dashboard
    @DynamicAttrs
    """
    def __repr__(self):
        return '<DashboardTab id=%s>' % self.id

class DashboardBlock(object):
    """
    Model for a Block within a dashboard.
    @DynamicAttrs
    """
    def __repr__(self):
        return '<DashboardBlock id=%s>' % self.id

class DashboardColumn(object):
    """
    Block contains Columns
    @DynamicAttrs
    """
    def _get_widgets(self):
        session = Session.object_session(self)
        return session.query(DashboardWidget).with_polymorphic('*').filter_by(column_id=self.id).order_by(
                            DashboardWidget.position).all()

    widgets = property(_get_widgets)

    def __repr__(self):
        return '<DashboardColumn id=%s>' % self.id

class DashboardWidget(object):
    """
    Model for the Dashboard Widget.
    @DynamicAttrs
    """

    def __repr__(self):
        return '<DashboardWidget id=%s>' % self.id

class DashboardWidgetCQ(DashboardWidget):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<DashboardWidgetCQ widget_id=%s>' % self.widget_id
