# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.dashboard.controllers import DashboardStateController,\
    ColumnStateController, TabStateController
from basesoft.web.contrib.dashboard.models import Dashboard
from basesoft.web.contrib.license.controllers import LicenseController
from django.utils.translation import ugettext as _
from basesoft.web.core.util.controllers import StateManager

from basesoft import logging

class DashboardRPCHandler:
    """
    Used for creation and updates of CLM Accounts. Linking of Geolevels and/or locales.
    """
    def __init__(self, request, **kwargs):
        self.request = request
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']
        self.manager = StateManager(None, self.account_id, Dashboard.MODE_COMPOSE)
        self.controller = DashboardStateController(self.sa_session, self.manager)

    def get_widgets(self):
        return self.controller.get_widgets(show_hidden=False)

    def get_dashboards(self):
        return [{'id': d.id, 'name': d.name, 'is_shared': d.is_shared, 'is_readonly': d.is_readonly, 'is_role_db': (len(d.roles) > 0)} for d in self.controller.get_dashboards()]

    def save(self, state, group_id=None):
        """
        Save the state to the current logged in account.
        TODO: implement multi-group support - #3173
        """
        return self.controller.save_state(state)

    def save_dashboard_name(self, id, name):
        return self.controller.save_name(id, name)

    def load(self, id=None):
        """
        Load the state of the current logged in account.
        """
        return self.controller.load_state(id)

    def load_tab(self, _id):
        return self.controller.tabStateController.load(_id)

    def get_sharing(self, dashboard_id):

        sharing = self.controller.get_sharing(dashboard_id)
        logger = logging.getLogger("basesoft.web.contrib.dashboard", request=self.request)
        logger.info('sharing: %s' % str(sharing))
        return sharing

    def save_sharing(self, dashboard_id, values):

        assert dashboard_id is not None

        dashboard = self.sa_session.query(Dashboard).get(dashboard_id)
        license_c = LicenseController(self.sa_session,account_id=self.controller.get_account())

        for group in license_c.get_groups_for_domain():

            viewable = values['viewable_'+str(group.id)]
            editable = values['editable_'+str(group.id)]
            account_ids = [int(a) for a in values['users_'+str(group.id)]]

            self.controller.save_sharing(dashboard, group, viewable, editable, account_ids)

    def get_copy_widget_options(self, current_dashboard_id=None):

        rv = {}

        for d in self.controller.get_dashboards():

            if d.is_readonly or (current_dashboard_id is not None and d.id == int(current_dashboard_id)):
                continue

            v = [{'label': t.label, 'value': t.id} for t in d.tabs]
            rv[d.name] = v

        return rv


    def copy_widget(self, widget_state, tab_ids):

        c = ColumnStateController(self.sa_session, self.manager)

        for tab_id in tab_ids:
            c.copy_widget_to_tab(widget_state, int(tab_id))


    def get_copy_tab_options(self, current_dashboard_id=None):
        return {_('Dashboards'): [{'label': d.name, 'value': d.id}
                                  for d in self.controller.get_dashboards()
                                  if not d.is_readonly and (current_dashboard_id is None or d.id != int(current_dashboard_id))]
        }

    def copy_tab(self, tab_state, dashboard_ids):

        c = TabStateController(self.sa_session, self.manager)

        for d_id in dashboard_ids:
            c.copy_tab_to_dasboard(tab_state, int(d_id))



handlers = (DashboardRPCHandler, )
