# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel

from basesoft.web.contrib.file.models import File, FileCategory, ImportFile, ImportFileType

class FileCustomQuery(CustomQuery):
    def tables(self):
        return (
            CustomQueryModel(File),
            CustomQueryModel(FileCategory, onclause=File.category_id==FileCategory.id),
        )

    def fields(self):
        return (
            CustomQueryField(File.id, _('ID'), key='id'),
            CustomQueryField(File.file_name, _('Name'), key='file_name'),
            CustomQueryField(File.file_hash, _('Hash'), key='file_hash'),
            CustomQueryField(File.polymorphic_type, _('Type')),
            CustomQueryField(FileCategory.id, _('Category'), in_select=False, is_filter=True, filter_options=FileCategory.label),
            CustomQueryField(FileCategory.label, _('Category'), is_group_by=True),
            CustomQueryField(File.created, _('Created'), is_sorted=True, sorted_asc=False),
        )

class AbstractFileImportCustomQuery(CustomQuery):

    from sqlalchemy.orm import aliased
    validationFile = aliased(File, name='validationFile')
    def tables(self):
        return (
            CustomQueryModel(ImportFile),
            CustomQueryModel(ImportFileType),
            CustomQueryModel(self.validationFile,
                             onclause=ImportFile.failure_file_id==self.validationFile.id, use_outerjoin=True),
        )
    def fields(self):
        return (
            CustomQueryField(ImportFile.id, _('ID'), key='id'),
            CustomQueryField(ImportFile.file_name, _('File name'), is_filter=True, key='file_name'),
            CustomQueryField(ImportFile.file_hash, _('File hash'), key='file_hash'),
            CustomQueryField(ImportFile.status, _('Status'), is_filter=True, key='status'),
            CustomQueryField(ImportFile.result, _('Comment'), key='result'),
            CustomQueryField(self.validationFile.file_name, _('Validation file'), is_filter=True, key='validation_file'),
            CustomQueryField(self.validationFile.file_hash, _('Validation file hash'), key='validation_file_hash'),
            CustomQueryField(ImportFile.created, _('Created'), key='created'),
            CustomQueryField(ImportFile.last_updated, _('Last updated'), key='last_updated'),
        )
    def set_where(self):
        self.where = ImportFileType.name==self.IMPORT_FILE_TYPE

    order_by = (ImportFile.id.desc(), )

custom_queries = (FileCustomQuery, )
