# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This file contains the method to remove any old unused(unprocessed) uploaded file.
It starts every hour and removes all unused uploaded files(in tblFileTmp) that are
older than 24 hours.
"""

import commands
import datetime

from django.conf import settings

from basesoft.web.contrib.file.controllers import UploadFileController
from basesoft.web.contrib.file.models import FileTmp
from basesoft.scheduler.task import ScheduleTask

from basesoft import logging
logger = logging.getLogger('basesoft.scheduler')

class RemoveTempFileTask(ScheduleTask):

    INTERVAL = 3600 * 4

    def execute(self):
        """
        Remove files from db (tblFileTmp) and disk that are older than 24 hours.
        """

        # delete all files that are older than 24h
        # http://www.howtogeek.com/howto/ubuntu/delete-files-older-than-x-days-on-linux/
        cmd = "/usr/bin/find %s -maxdepth 1 -mtime +1 -exec rm {} \;" % settings.TMP_FILE_REPOSITORY
        status, output = commands.getstatusoutput(cmd)
        if status != 0:
            raise Exception('error while deleting files: %s' % output)

        # get all files older than 24 hours
        files = self.sa_session.query(FileTmp).filter(FileTmp.created<=datetime.datetime.now()-datetime.timedelta(days=1)).all()
        if len(files):
            logger.info('RemoveUploadedTempFileTask: removing %s uploaded temp files' % len(files))

            # remove them using the controller method
            controller = UploadFileController(self.sa_session, None)
            controller.delete_temp_files(files)

        return None

# publish task so the scheduler can pick them up
tasks = (RemoveTempFileTask,)