# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.graph.controllers import GraphController

class GraphRPCHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.user_id = request.session['user_id']
        self.gc = GraphController(self.sa_session, account_id=self.user_id, request=request)

    def get_available_color_sets(self):
        """
        Get the list of color sets that are available for the current user based on his license
        """
        default_color_set_preference = self.gc.config.get('basesoft.web.contrib.graph.default_color_set')
        available_color_sets = [{
            'label': e.name,
            'value': e.name,
            'default': e.name == default_color_set_preference
        } for e in self.gc.get_color_sets()]

        return available_color_sets

    def get_available_graph_themes(self):
        """
        Get the list of graph themes that are available for the current user based on his license
        """
        return [{'label': e.name, 'value': e.id} for e in self.gc.get_graph_themes()]

    def get_graph_theme_css(self, graph_theme):
        """
        Return the class of the given graph_theme
        @param graph_theme: the id of the theme
        @return: the css class or None
        """
        if not graph_theme:
            return None

        try:
            gt = self.gc.get_graph_themes(int(graph_theme))[0]
            return gt.css_class
        except IndexError:
            return None

handlers = (GraphRPCHandler, )
