# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from django.template import RequestContext
from django.shortcuts import render_to_response

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.contrib.home.controllers import HomeController
from basesoft.web.contrib.license.controllers import LicenseController

from django.conf import settings
from basesoft.web.contrib.branding.controllers import LicenseThemeController
from basesoft.web.core.exceptions import PublicInactiveAccountException
from basesoft.web.contrib.account.controllers import AccountController

def rpc(request):

    dispatcher = JsonRpcDispatcher()

    class Handler:

        def __init__(self, request):
            self.request = request

        def get_home_page(self):

            account_id = None
            if self.request.session.has_key('user_id'):
                account_id = self.request.session['user_id']

            hc = HomeController(self.request.Session(),account_id=account_id, request=request)
            return hc.get_home_application()

    dispatcher.register_instance(Handler(request))

    return dispatcher.handle_request(request)


def home(request):
    """
    """

    from basesoft.web.contrib.i18n.controllers import I18nController
    from basesoft.web.contrib.i18n.models import Locale

    user_id = None
    if request.session.has_key('user_id'):
        user_id = request.session['user_id']
    session = request.Session()

    try:
        ltc = LicenseThemeController(session, account_id=user_id, request=request)
    except PublicInactiveAccountException:
        user_id = None
        ac = AccountController(session, account_id=user_id, request=request)
        ac.logout(request)
        ltc = LicenseThemeController(session, account_id=user_id, request=request)
    current_theme = ltc.get_current_theme()
    theme_css = current_theme.css_class
    theme_module_name = current_theme.module_name
    request.session['theme_module_name'] = theme_module_name

    title = ''
    if current_theme.title:
        title = current_theme.title

    domain = None
    if user_id is None:

        lc = LicenseController(session)
        site_licensee = lc.get_license_for_request(request)
        domain = site_licensee.domain.id

    controller = I18nController(session, account_id=user_id)
    locales = [(l.locale, l.name) for l in controller.get_domain_locales(domain_id=domain,
                                                                         availability=Locale.AVAILABLE_FOR_SITE_TRANSLATION)]

    return render_to_response('home.html', {
        'VERSION': settings.VERSION,
        'theme_css': theme_css,
        'theme_module_name': theme_module_name,
        'home_title': title,
        'locales': locales
    },context_instance=RequestContext(request))
