'''
Created on 11 May 2011

@author: martijn
'''
from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQueryModel,\
    CustomQueryField, CustomQuery, DATA_TYPE_BOOLEAN
from basesoft.web.contrib.i18n.models import Country, DomainCountry, Language, DomainLanguage

from sqlalchemy.sql.expression import and_, func
from sqlalchemy.orm import outerjoin


class DomainCountryCustomQueryModel(CustomQueryModel):

    def __init__(self, onclause=None, priv=True, use_outerjoin=False, *args, **kwargs):
        CustomQueryModel.__init__(self, Country, onclause, priv, use_outerjoin, *args, **kwargs)

    def process(self, custom_query, fo, f, initial):

        domain = custom_query.get_default_group().license.domain

        fo, f, initial = CustomQueryModel.process(self, custom_query, fo, f, initial)
        fo = outerjoin(fo, DomainCountry, onclause=(and_(Country.id == DomainCountry.country_id, DomainCountry.domain_id == domain.id)))

        return fo, f, initial



class DomainCountryCustomQueryField(CustomQueryField):

    def __init__(self, field, label, *args, **kwargs):


        if field is None:
            field = func.ifnull(DomainCountry.name, Country.name)

        assert not kwargs.has_key('filter_options')
        if not kwargs.has_key('type') or  kwargs['type'] != DATA_TYPE_BOOLEAN:
            if kwargs.has_key('is_filter') and kwargs['is_filter']:
                kwargs['filter_options'] = func.ifnull(DomainCountry.name, Country.name)

        CustomQueryField.__init__(self, field, label, *args, **kwargs)

class DomainCountryCustomQuery(CustomQuery):

    permissions = ['manage_domain_country_mapping']

    def tables(self):
        return (
            DomainCountryCustomQueryModel(),
        )

    def fields(self):
        return (
            CustomQueryField(Country.id, 'ID', key='country_id'),
            CustomQueryField(Country.name, _('Original name'), is_filter=True, key='original'),
            DomainCountryCustomQueryField(None, _('Local name'), is_filter=True, key='country'),
            DomainCountryCustomQueryField(func.ifnull(DomainCountry.active, True), _('Active'), is_filter=True, key='is_active', type=DATA_TYPE_BOOLEAN),
            CustomQueryField(Country.iso_3166_a2, 'a2', is_filter=True),
            CustomQueryField(Country.iso_3166_a3, 'a3', is_filter=True),
        )

class DomainLanguageCustomQuery(CustomQuery):

    permissions = ['manage_domain_language_aliases']

    def tables(self):
        domain = self.get_default_group().license.domain

        return (
            CustomQueryModel(Language),
            CustomQueryModel(DomainLanguage, use_outerjoin=True, onclause=and_(Language.id==DomainLanguage.language_id, DomainLanguage.domain==domain)),
        )

    def fields(self):
        return (
            CustomQueryField(Language.id, 'ID', key='language_id'),
            CustomQueryField(Language.name, _('Original name'), is_filter=True, key='original'),
            CustomQueryField(Language.iso_639_1, 'a2', is_filter=True, key='iso_639_1'),
            CustomQueryField(func.group_concat(' ', DomainLanguage.name), _('Aliases'), key='alias'),
        )

    group_by = (Language.id, )

custom_queries = (DomainCountryCustomQuery, DomainLanguageCustomQuery, )

