# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

# List of current EU countries.
EU_COUNTRIES = ('BE', 'BG', 'CZ', 'DK', 'DE', 'EE', 'IE', 'ES', 'FR', 'GB', 'GR', 'HR', 'IT', 'CY', 'LV', 'LT', 'LU', 'HU', 'MT', 'NL', 'AT', 'PL', 'PT', 'RO', 'SI', 'SK', 'FI', 'SE', )

# European Free Trade Association (http://en.wikipedia.org/wiki/European_Free_Trade_Association)
EFTA_COUNTRIES = [cc for cc in EU_COUNTRIES] + ['LI', 'NO', 'CH', 'IS']

class Locale(object):
    """
    Model representing a locale.
    @DynamicAttrs
    """
    NOT_AVAILABLE = 0
    AVAILABLE_FOR_COMPOSE = 1
    AVAILABLE_FOR_SITE_TRANSLATION = 2

    def __init__(self, name, locale):
        """
        @param name: C{String} - name of the locale
        @param locale: C{String} - code of the locale (5 chars)
        """
        self.name = name
        self.locale = locale

    def get_state(self):
        """
        Return the state of the locale
        """
        return {
            'id': self.id,
            'name': self.name,
            'locale': self.locale,
            'direction': 'ltr', # FIXME: add directions for the regular locales too
        }
    def __repr__(self):
        return '<Locale if=%s>' % self.id

class DomainLocale(object):
    """
    @DynamicAttrs
    """

    def __repr__(self):
        return '<DomainLocale locale=%s>' % self.locale

class Country(object):
    """
    Model representing a country.
    @DynamicAttrs
    """

    cols_i18n = {
        "name": _("Country name"),
        "iso_3166_a2": _("Country code (iso_3166_a2)"),
        "iso_3166_a3": _("Country code (iso_3166_a3)")
    }

    def __init__(self, name, iso_3166_a2, iso_3166_a3):
        """
        @param name: C{String} - country name
        @param iso_3166_a2: C{String} - country code (2 chars)
        @param iso_3166_a3: C{String} - country code (3 chars)
        """
        self.name = name
        self.iso_3166_a2 = iso_3166_a2
        self.iso_3166_a3 = iso_3166_a3

    def __repr__(self):
        return '<Country code="%s">' % self.iso_3166_a2

class DomainCountry(object):
    """
    @DynamicAttrs
    """

    def __repr__(self):
        return '<DomainCountry code="%s">' % self.country.iso_3166_a2

class Language(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<Language code="%s">' % self.iso_639_1

class DomainLanguage(object):
    """
    @DynamicAttrs
    """

    def __repr__(self):
        return '<DomainLanguage code="%s">' % self.language.iso_639_1

