# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.contrib.i18n.controllers import I18nController
from basesoft.web.contrib.i18n.models import Locale

def i18n_rpc(request):
    """
    Contains methods for getting i18n related things: locales, countries.
    """

    sa_session = request.Session()
    account_id  = None
    locale = None
    if request.session.has_key('user_id'):
        account_id = request.session['user_id']
    if request.session.has_key('config'):
        locale = request.session['config']['basesoft.web.contrib.locale']
    controller = I18nController(sa_session, locale, account_id)

    def get_countries():
        """
        Return list with countries.
        """
        return [{'label': country.name if domain_country is None else domain_country.name, 'value': country.id, 'default': False} for country, domain_country in controller.get_domain_countries()]

    def get_locales():
        """
        Return list with available locales for compose.
        """
        return [{'label': locale.name,
                 'locale': locale.locale,
                 'value': locale.locale,
                 'default': False,
                 'id': locale.id}
                    for locale in controller.get_domain_locales(availability=Locale.AVAILABLE_FOR_COMPOSE)]

    def get_site_translations_locales():
        """
        Return list with available locales for site translations.
        """
        return [{'label': locale.name,
                 'locale': locale.locale,
                 'value': locale.locale,
                 'default': False}
                    for locale in controller.get_domain_locales(availability=Locale.AVAILABLE_FOR_SITE_TRANSLATION)]

    def get_all_locales():
        return [{'label': locale.name, 'locale': locale.locale, 'value': locale.id, 'default': False}
                for locale in controller.get_locales()]

    def get_domain_locales_data(domain_id):
        return {'all_locales': get_all_locales(),
                'compose_locales': [l.id for l in controller.get_domain_locales(domain_id=domain_id,
                                                                                availability=Locale.AVAILABLE_FOR_COMPOSE)],
                'translation_locales': [l.id for l in controller.get_domain_locales(domain_id=domain_id,
                                                                                    availability=Locale.AVAILABLE_FOR_SITE_TRANSLATION)]}


    def save_domain_locales(domain_id, locales):
        controller.save_or_update_domain_locales(domain_id, locales)
        return True

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_function(get_countries)
    dispatcher.register_function(get_locales)
    dispatcher.register_function(get_site_translations_locales)
    dispatcher.register_function(get_all_locales)
    dispatcher.register_function(get_domain_locales_data)
    dispatcher.register_function(save_domain_locales)
    return dispatcher.handle_request(request)
