# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel

from basesoft.web.contrib.mailing.models import Mail
from basesoft.web.core.util.controllers import get_models_of_type

class MailCustomQuery(CustomQuery):
    permissions = ['view_mail']
    _mail_types = None

    def tables(self):
        return (
            CustomQueryModel(Mail),
        )
    def fields(self):
        return (
            CustomQueryField(Mail.id, _('ID')),
            CustomQueryField(Mail.name, _('Name'), is_filter=True),
            CustomQueryField(Mail.id, _('Translate'), key='translate'),
            CustomQueryField(Mail.polymorphic_type, _('Type'), key='polymorphic_type', is_group_by=True, is_filter=True, filter_options=Mail.polymorphic_type),
        )

    def set_value_processors(self):

        if self._mail_types is None:
            self._mail_types = {}
            mails = get_models_of_type(Mail, True)
            for mail in mails:
                self._mail_types[mail['value']] = mail['label']

        def cb(row, key):
            pt = getattr(row, key)
            if not self._mail_types.has_key(pt):
                return pt
            return self._mail_types[pt]

        self._value_processors['polymorphic_type'] = cb

custom_queries = (MailCustomQuery, )
