# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class ManualChapterManagerIcon(IconBarIcon):
    """
    Class responsible for handling the manual chapter manager icon
    """

    id = 'icon-manual-manager'
    title = _('Manual Chapter manager')
    icon_index = 75
    component = 'iq_manual_manager'
    group = 'manual'

    char_main = u'\uE80c' # Question mark icon
    char_sub = u'\uE806' # Head icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        """
        controller.assert_permissions('create_manual_chapter')

class ManualIndexIcon(IconBarIcon):
    """
    Class responsible for handling the manual viewer icon
    """

    id = 'icon-manual-index'
    title = _('Manual Index')
    icon_index = 74
    component = 'iq_manual_index'
    group = 'manual'

    char_main = u'\uE80c' # Question mark icon
    char_sub = u'\uE802' # List icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        """
        controller.assert_permissions('view_manual_chapter')

def get_icons():
    return [ManualChapterManagerIcon(), ManualIndexIcon(),]
