# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from basesoft.web.contrib.manual.controllers import ManualController

class ManualRpcHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.request = request
        account_id = None

        if request.session.has_key('user_id'):
            account_id = request.session['user_id']

        self._controller = ManualController(self.sa_session, account_id=account_id)

    def get_applications(self):
        return [{'value': i, 'label': i} for i in settings.INSTALLED_APPS]

    def get_by_application_and_handle(self, app, handle):
        return self._controller.get_by_application_and_handle(app, handle)

    def get_chapter_state(self, id, verify=True):
        return self._controller.get_chapter_state(id, verify=verify)

    def get_chapter_translation(self, id, locale, subchapters=True):
        return self._controller.get_full_chapter(id, locale)

    def get_chapter_hierachy(self):
        return self._controller.get_chapter_hierachy()

    def get_chapters(self):
        return [{'value': c.id, 'label': '%s - %s' % (c.app, c.handle)} for c in self._controller.get_chapters()]

    def save_or_update_manual_chapter(self, arguments):
        chapter = self._controller.save_or_update_manual_chapter(**arguments)
        return chapter.id

    def save_chapter_translation(self, id, values):
        self._controller.save_chapter_translation(id, values)
        return True

    def generate_manual_chapters(self, arguments, overwrite_translations=False):
        chapter = self._controller.generate_manual_chapters(overwrite_translations=overwrite_translations, **arguments)
        return chapter.id

handlers = (ManualRpcHandler, )
