# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _
from sqlalchemy import and_, func, case, or_

from basesoft.web.core.exceptions import PermissionDeniedException
from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel

from basesoft.web.contrib.account.models import Account
from basesoft.web.contrib.contact.models import Person
from basesoft.web.contrib.notification.models import Notification, NotificationAccount

class NotificationCustomQuery(CustomQuery):
    def validate_access(self):
        if not (self.has_permissions('view_notification')):
            raise PermissionDeniedException('You do not have access to the requested resource')

    def tables(self):
        return (
            CustomQueryModel(Notification),
            CustomQueryModel(NotificationAccount, onclause=and_(Notification.id==NotificationAccount.notification_id,
                                                                NotificationAccount.account_id==self.account_id
                                                    ),
                             use_outerjoin=True),
            CustomQueryModel(Account, onclause=Notification.sender_id==Account.id, use_outerjoin=True),
            CustomQueryModel(Person, onclause=Account.person_id==Person.id, use_outerjoin=True)
        )

    def fields(self):
        return (
            CustomQueryField(Notification.id, _('ID'), key='id'),
            CustomQueryField(case([(Notification.sender!=None,
                                    case([(Account.person==None, func.concat("(", Account.name,")"))],
                                         else_=func.concat(case([(Person.first_name!=None, Person.first_name)], else_=""),
                                                      " ",
                                                      case([(Person.middle_name!=None, Person.middle_name)], else_=""),
                                                      " ",
                                                      case([(Person.last_name!=None, Person.last_name)], else_=""),
                                                      )
                                         )
                                 )], else_=_("System")),
                             _('From'), key='sender', is_filter=True),
            CustomQueryField(case([(func.length(Notification.subject)<=40, Notification.subject)],
                                  else_=func.substring(Notification.subject, 1, 40)+" ..."),
                             _('Subject'), key='subject', is_filter=False),
            CustomQueryField(Notification.subject, _('Subject'), key='subject_full', is_filter=True, in_select=False),
            CustomQueryField(Notification.created, _('Send date'), key='created'),
            CustomQueryField(case([(Notification.id==NotificationAccount.notification_id, 0)], else_=1),
                             _('Status'), key='status', is_filter=True, filter_options=[{
                                'value': 1,
                                'label': _("Unread"),
                            },{
                                'value': 0,
                                'label': _("Read"),
                            }]),
        )
    order_by = (Notification.created.desc(), )
    group_by = (Notification.id, )

    def set_where(self):
        self.where = or_(NotificationAccount.archived==False, NotificationAccount.archived==None)

    # in this case we don't need filters for deleted and archived on the Notification object
    def _set_deleted_and_archived_fields(self):
        pass

custom_queries = (NotificationCustomQuery, )
