# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.notification.controllers import NotificationController
from basesoft.web.contrib.account.decorators import login_required

class NotificationRPCHandler:
    @login_required
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.controller = NotificationController(request.Session(), account_id=request.session['user_id'])

    def load_notification(self, notification_id, mark_as_read=True):
        return self.controller.load_notification(notification_id, mark_as_read)

    def get_available_recipients_groups(self):
        return self.controller.get_available_recipients_groups()

    def create_notification(self, subject, text, recipients, for_group, for_license, for_domain):
        n = self.controller.set_notification(subject, text)
        return self.controller.create_notification(n, recipients, for_group, for_license, for_domain).id

    def _set_notification_from_state(self, state):
        return self.controller.set_notification(state['subject'], state['text'])

    def create_notification_from_state(self, state):
        if state['everybody']:
            recipients = self.controller.get_all_recipients()
        else:
            recipients = state['recipients']
            for r in recipients: #for_group is not in state from client
                r['for_group'] = True
                if not r.has_key('for_license'):
                    r['for_license'] = False
                if not r.has_key('for_domain'):
                    r['for_domain'] = False
        n = self._set_notification_from_state(state)
        #create notification for first recipient, then set view privs for the rest
        notification = self.controller.create_notification(n, recipients[0]['group'],
                        recipients[0]['for_group'], recipients[0]['for_license'], recipients[0]['for_domain'])
        i = 0
        for r in recipients:
            #skip first because it was already added
            if i == 0:
                continue
            i = 1
            if r['group'] == '' or r['group'] is None:
                continue
            g = self.has_recipient_permissions(r['group'], r['for_license'], r['for_domain'])
            if g:
                notification = self.save_view_privileges(notification, g, r['for_group'], r['for_license'], r['for_domain'])
        return notification.id

    def mark_as_archived(self, notifications):
        self.controller.mark_as_archived(notifications)

    def get_icon_count(self):
        """
        Return the number of unread notifications so the iconbar can show the number next to the icon, or None if we
        don't have any unread notifications, which means the icon number won't be shown
        """
        icon_count = self.controller.get_unread_notifications_count()
        if icon_count == 0:
            return None

        return icon_count

class SystemNotificationRPCHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        account_id = None
        if request.session.has_key('user_id'):
            account_id = request.session['user_id']
        self.controller = NotificationController(request.Session(), account_id=account_id)

    def create_system_notification(self, subject, text, recipients, for_group, for_license, for_domain):
        n = self.controller.set_notification(subject, text)
        return self.controller.create_system_notification(n, recipients, for_group, for_license, for_domain).id

handlers = (NotificationRPCHandler, SystemNotificationRPCHandler)


