'''
Created on 24 Oct 2013

@author: martijn
'''
from basesoft.web.contrib.order.models import Currency, CountryCurrency
from sqlalchemy.orm.exc import NoResultFound
from basesoft.web.contrib.i18n.models import Country

from logging import getLogger
from basesoft.web.contrib.order.configurable_widgets.order_result import OrderResultWidget
logger = getLogger()

CURRENCY_SYMBOLS = {
    'EUR': u"\u20AC".encode('utf8'),
    'GBP': u"\u00A3".encode('utf8'),
    'USD': u'$'
}


configurable_widgets = (OrderResultWidget, )

def post_sync(sa_session, app_name):

    # sync currencies when countries are available
    if app_name == 'basesoft.web.contrib.i18n':

        import iqportal
        import os
        import csv
        import datetime

        # csv file is generated from http://www.commondatahub.com/live/geography/currency/country_currency_codes
        # all rows with comma delimited, single quotes, no header and all columns
        csv_file = os.path.join(os.path.dirname(__file__), 'currencies.txt')

        with open(csv_file, 'rb') as f:
            reader = csv.reader(f, delimiter=',', quotechar="'")
            for row in reader:

                currency_name = row[1]
                country_char3_code = row[3]
                currency_char3_code = row[5]
                effective_date = None if row[9] == '' else datetime.datetime.strptime(row[9], "%m/%d/%Y")
                expiration_date =None if row[10] == '' else  datetime.datetime.strptime(row[10], "%m/%d/%Y")

                try:
                    currency = sa_session.query(Currency).filter(Currency.code==currency_char3_code).one()
                except NoResultFound:
                    currency = Currency()
                    currency.code = currency_char3_code

                if CURRENCY_SYMBOLS.has_key(currency_char3_code):
                    currency.symbol = CURRENCY_SYMBOLS[currency_char3_code]
                currency.name = currency_name
                currency.effective_date = effective_date
                currency.expiration_date = expiration_date

                try:
                    country = sa_session.query(Country).filter(Country.iso_3166_a3 == country_char3_code).one()
                except NoResultFound:
                    logger.warn("Country not found for currency country %s" % country_char3_code)
                    continue

                # add this pont add the currency, making sure there is a country for this currency
                sa_session.add(currency)

                # insert currency/country combination
                try:
                    country_currency = sa_session.query(CountryCurrency).filter(CountryCurrency.country == country, CountryCurrency.currency == currency).one()
                except NoResultFound:
                    country_currency = CountryCurrency()
                    country_currency.country = country
                    country_currency.currency = currency

                sa_session.add(country_currency)

        sa_session.flush()

def procedures():

    return [

        """
        DROP FUNCTION IF EXISTS `iqGetCurrencyIdForCountry`
        """,
        """
        CREATE FUNCTION `iqGetCurrencyIdForCountry`(
            in_country_id INT,
            in_date DATE )
            RETURNS INT
            READS SQL DATA
        BEGIN
            DECLARE currency_id INT;
            SET currency_id = NULL;

            SELECT c.id INTO currency_id
            FROM tblCurrency c
            INNER JOIN tblCountryCurrency cc ON c.id = cc.currency_id
            WHERE cc.country_id = in_country_id
            AND (effective_date is null or effective_date <= in_date)
            AND (expiration_date is null or expiration_date > in_date)
            ORDER BY expiration_date DESC
            LIMIT 1;

            RETURN(currency_id);

        END
        """,

        """
        DROP FUNCTION IF EXISTS `iqGetCurrencyCodeForCountry`
        """,
        """
        CREATE FUNCTION `iqGetCurrencyCodeForCountry`(
            in_country_id INT,
            in_date DATE )
            RETURNS VARCHAR(3)
            READS SQL DATA
        BEGIN
            DECLARE currency_code VARCHAR(3);
            SET currency_code = NULL;

            SELECT c.code INTO currency_code
            FROM tblCurrency c WHERE c.id = iqGetCurrencyIdForCountry(in_country_id, in_date);

            RETURN(currency_code);

        END
        """,

        """
        DROP FUNCTION IF EXISTS `iqGetCurrentTaxCountryRate`
        """,
        """
        CREATE FUNCTION `iqGetCurrentTaxCountryRate`(
            in_tax_class_id INT,
            in_country_id INT,
            in_start_date DATE )
            RETURNS DECIMAL(10,5)
            READS SQL DATA
        BEGIN
            DECLARE rate DECIMAL(10,5);
            SET rate = NULL;

            SELECT tccr.rate INTO rate
            FROM tblTaxClassCountryRate tccr
            INNER JOIN tblTaxClassCountry tcc ON tccr.tax_class_country_id = tcc.id
            WHERE
                tcc.`tax_class_id` = in_tax_class_id AND
                tcc.country_id = in_country_id AND
                tccr.start_date <= in_start_date
            ORDER BY tccr.`start_date` DESC
            LIMIT 1;

            RETURN(rate);

        END
        """,

        """
        DROP FUNCTION IF EXISTS `iqGetCurrentOrderPaymentStatus`
        """,
        """
        CREATE FUNCTION `iqGetCurrentOrderPaymentStatus`(
            in_order_id INT )
            RETURNS INT
            READS SQL DATA
        BEGIN
            DECLARE status INT;

            -- default status is pending, this will be returned if the query below
            -- does not select any rows
            SET status = 0;

            -- if order is submitted (status 3) and there is no payment status and the total_amount_including_vat is 0
            -- the payment status should be set as completed (3).
            SELECT IFNULL(ops.status, IF(o.status = 3 and o.total_amount_including_vat = 0, 3, 0)) INTO status
            FROM tblOrder o
            LEFT JOIN tblOrderPaymentStatus ops
                ON o.id = ops.order_id
            WHERE o.id = in_order_id
            ORDER BY ops.last_updated DESC
            LIMIT 1;
            RETURN(status);
        END
        """,

        """
        DROP FUNCTION IF EXISTS `iqGetCurrentOrderPaymentMethod`
        """,
        """
        CREATE FUNCTION `iqGetCurrentOrderPaymentMethod`(
            in_order_id INT )
            RETURNS VARCHAR(255)
            READS SQL DATA
        BEGIN
            DECLARE label VARCHAR(255);

            SET label = '-';

            SELECT pm.label INTO label
            FROM tblOrder o
            INNER JOIN tblOrderPaymentStatus ops
                ON o.id = ops.order_id
            INNER JOIN tblPaymentMethod pm
                ON ops.payment_method_id = pm.id
            WHERE o.id = in_order_id
            ORDER BY ops.last_updated DESC
            LIMIT 1;

            RETURN(label);
        END
        """,

        """
        DROP FUNCTION IF EXISTS `iqGetCurrentOrderPaymentDate`
        """,
        """
        CREATE FUNCTION `iqGetCurrentOrderPaymentDate`(
            in_order_id INT )
            RETURNS DATETIME
            READS SQL DATA
        BEGIN
            DECLARE payment_date DATETIME;

            SET payment_date = NULL;

            SELECT ops.payment_date INTO payment_date
            FROM tblOrder o
            LEFT JOIN tblOrderPaymentStatus ops
                ON o.id = ops.order_id
            WHERE o.id = in_order_id
            ORDER BY ops.last_updated DESC
            LIMIT 1;

            RETURN(payment_date);
        END
        """,

    ]
