# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class TaxClassManagerIcon(IconBarIcon):

    id = 'icon-tax-manager'
    title = _('Tax class manager')
    icon_index = 75
    component = 'iq_order_tax_class_manager'
    group = 'financial'

    char_main = u'\uE808'

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('manage_tax_classes')

class TaxClassCountryManagerIcon(IconBarIcon):

    id = 'icon-tax-manager'
    title = _('Tax country manager')
    icon_index = 75
    component = 'iq_order_country_tax_manager'
    group = 'financial'

    char_main = u'\uE800'
    char_sub = u'\uE808'

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('view_country_tax')

class GLAccountManagerIcon(IconBarIcon):

    id = 'icon-tax-manager'
    title = _('GL Account manager')
    icon_index = 75
    component = 'iq_order_gl_account_manager'
    group = 'financial'

    char_main = u'\uE809'
    char_sub = u'\uE806'

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('manage_gl_accounts')


def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """
    return [TaxClassManagerIcon(), TaxClassCountryManagerIcon(), GLAccountManagerIcon()]
