'''
Created on Oct 2, 2013

@author: marten
'''
import datetime

from sqlalchemy.orm import mapper, relation
from sqlalchemy import Integer, Column, String, ForeignKey, SmallInteger, DateTime, UniqueConstraint, Numeric, UnicodeText, Date, DECIMAL

from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.order import models as order_models
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.i18n.models import Country
from basesoft.web.contrib.order.models import TaxClass, TaxClassCountry

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    """
    This function is called by Django middleware when needed.
    """
    global __tables_defined
    if __tables_defined:
        return

    aggOrder = IQTable('aggOrder', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('order_id', Integer, index=True, nullable=False),
        Column('order_line_id', Integer, index=True, nullable=False),
        Column('order_type', String(64), index=True, nullable=False),
        Column('order_status', SmallInteger, nullable=False),
        Column('invoice_type', String(32), index=True, nullable=False),
        Column('merchant_id', Integer, index=True, nullable=False),
        Column('client_id', Integer, index=True, nullable=True),
        Column('submitted_time_id', Integer, index=True, nullable=True),
        Column('payment_status', Integer, index=True, nullable=False),
        Column('payment_method', String(64), index=True, nullable=False),
        Column('payment_time_id', Integer, index=True, nullable=True),
        Column('gl_account', String(64), index=True, nullable=False),
        Column('quantity', Integer, index=True, nullable=True),
        Column('line_amount_excluding_vat', DECIMAL(20,5), index=True, nullable=True),
        Column('line_amount_including_vat', DECIMAL(20,5), index=True, nullable=True),
        Column('price_per_unit', DECIMAL(20,5), index=True, nullable=True),
        Column('last_updated', DateTime(), index=True, nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )

    # CurrencyCode

    tblCurrency = IQTable('tblCurrency', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('code', String(3), nullable=False),
        Column('name', String(255), nullable=False),
        Column('symbol', String(3), nullable=True),
        Column('effective_date', DateTime(), nullable=True),
        Column('expiration_date', DateTime(), nullable=True),
        UniqueConstraint('code'),
    )

    tblCountryCurrency = IQTable('tblCountryCurrency', metadata,
        Column('country_id', Integer, ForeignKey('tblCountry.id', ondelete='CASCADE'), nullable=False, primary_key=True, ),
        Column('currency_id', Integer, ForeignKey('tblCurrency.id', ondelete='RESTRICT'), nullable=False, primary_key=True, ),
        UniqueConstraint('country_id', 'currency_id'),
    )

    # Tax

    tblTaxClass = IQTable('tblTaxClass', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('code', String(255), nullable=False),
        Column('name', String(255), nullable=False), # could be refactored to use locale
        Column('deleted', SmallInteger, nullable=False, default=0),
        Column('archived', SmallInteger, nullable=False, default=0),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )
    tblTaxClassGroup = IQTable('tblTaxClassGroup', metadata,
        Column('tax_class_id', Integer, ForeignKey('tblTaxClass.id', ondelete='CASCADE'), primary_key=True),
        Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
        Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
        Column('group_has_privilege', SmallInteger, default=0),
        Column('license_has_privilege', SmallInteger, default=0),
        Column('domain_has_privilege', SmallInteger, default=0),
        Column('world_has_privilege', SmallInteger, default=0),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
    )

    tblTaxClassCountry = IQTable('tblTaxClassCountry', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('tax_class_id', Integer, ForeignKey('tblTaxClass.id', ondelete='RESTRICT'), nullable=False),
        Column('country_id', Integer, ForeignKey('tblCountry.id', ondelete='RESTRICT'), nullable=False),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        UniqueConstraint('country_id', 'tax_class_id'),
    )

    tblTaxClassCountryRate = IQTable('tblTaxClassCountryRate', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('tax_class_country_id', Integer, ForeignKey('tblTaxClassCountry.id', ondelete='CASCADE'), nullable=False),
        Column('rate', Numeric(10, 5), nullable=False),
        Column('start_date', Date(), nullable=False),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        UniqueConstraint('tax_class_country_id', 'start_date'),
    )

    # GL Accounts

    tblGLAccount = IQTable('tblGLAccount', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('merchant_id', Integer, ForeignKey('tblLicense.id'), nullable=False),
        Column('code', String(255), nullable=False),
        Column('account_number', String(255), nullable=False),
        Column('deleted', SmallInteger, nullable=False, default=0),
        Column('archived', SmallInteger, nullable=False, default=0),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),

        UniqueConstraint('merchant_id', 'code'),
    )

    # Invoice
    tblInvoiceFile = IQTable('tblInvoiceFile', metadata,
        Column('file_id', Integer, ForeignKey('tblFile.id', ondelete='CASCADE'), primary_key=True),
        Column('is_credit', SmallInteger, nullable=False, default=order_models.InvoiceFile.IS_INVOICE),
    )

    tblNewInvoiceMail = IQTable('tblNewInvoiceMail', metadata,
        Column('mail_id', Integer, ForeignKey('tblMail.id', ondelete='CASCADE'), primary_key=True),
        Column('fallback_locale_id', Integer, ForeignKey('tblLocale.id'), nullable=False),
    )


    # PaymentMethod (abstract)
    tblPaymentMethod = IQTable('tblPaymentMethod', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('merchant_id', Integer, ForeignKey('tblLicense.id'), nullable=False),
        Column('label', UnicodeText, nullable=False),
        Column('priority', Integer, nullable=False, default=1),
        Column('polymorphic_type', String(255), nullable=False),
        Column('deleted', SmallInteger, nullable=False, default=0),
        Column('archived', SmallInteger, nullable=False, default=0),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
    )

    tblPaymentMethodBankTransfer = IQTable('tblPaymentMethodBankTransfer', metadata,
        Column('payment_method_id', Integer, ForeignKey('tblPaymentMethod.id', ondelete='CASCADE'), primary_key=True),
    )

    tblPaymentMethodOgone = IQTable('tblPaymentMethodOgone', metadata,
        Column('payment_method_id', Integer, ForeignKey('tblPaymentMethod.id', ondelete='CASCADE'), primary_key=True),
        Column('psp_id', String(255), nullable=False), # psp_id column is also used to map against settings which contains secret
        Column('sha_in', String(255), nullable=False),
        Column('sha_out', String(255), nullable=False),
    )

    # Order

    tblOrder = IQTable('tblOrder', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('polymorphic_type', String(255), nullable=False, default="Order"),
        Column('parent_order_id', ForeignKey('tblOrder.id'), nullable=True, default=None, unique=True),
        Column('type', SmallInteger, nullable=False, default=order_models.Order.TYPE_DEBIT),
        Column('client_id', ForeignKey('tblLicense.id'), nullable=False),
        Column('merchant_id', ForeignKey('tblLicense.id'), nullable=False),
        Column('status', SmallInteger, nullable=False, default=order_models.Order.STATUS_NEW),
        Column('currency_id', Integer, ForeignKey('tblCurrency.id', ondelete='RESTRICT'), nullable=False, ),
        Column('total_amount_excluding_vat', Numeric(20, 5), nullable=True),
        Column('total_amount_including_vat', Numeric(20, 5), nullable=True),
        Column('invoiced_amount', Numeric(20, 5), nullable=True),
        Column('invoice_id', ForeignKey('tblInvoiceFile.file_id'), nullable=True),
        Column('invoice_stage', SmallInteger, nullable=True),
        Column('auto_submitted', SmallInteger, nullable=True, default=None),
        Column('deleted', SmallInteger, nullable=False, default=0),
        Column('archived', SmallInteger, nullable=False, default=0),
        Column('export_date', DateTime(), nullable=True),
        Column('display_date', DateTime(), nullable=True),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
    )


    tblOrderGroup = IQTable('tblOrderGroup', metadata,
        Column('order_id', Integer, ForeignKey('tblOrder.id', ondelete='CASCADE'), primary_key=True),
        Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
        Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
        Column('group_has_privilege', SmallInteger, default=0),
        Column('license_has_privilege', SmallInteger, default=0),
        Column('domain_has_privilege', SmallInteger, default=0),
        Column('world_has_privilege', SmallInteger, default=0),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
    )

    tblOrderLine = IQTable('tblOrderLine', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('index', Integer, nullable=False),
        Column('polymorphic_type', String(255), nullable=False, default="OrderLine"),
        Column('order_id', Integer, ForeignKey('tblOrder.id', ondelete='CASCADE'), nullable=False),
        Column('description', UnicodeText, nullable=False),
        Column('quantity', Integer, nullable=False),
        Column('price_per_unit', Numeric(10, 5), nullable=False),
        Column('vat_rate', Numeric(10, 5), nullable=True),
        Column('gl_account_number', String(255), nullable=True),
        Column('line_amount_excluding_vat', Numeric(20, 5), nullable=False),
        Column('line_amount_including_vat', Numeric(20, 5), nullable=True),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        UniqueConstraint('index', 'order_id')
    )

    tblOrderPaymentStatus = IQTable('tblOrderPaymentStatus', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('order_id', Integer, ForeignKey('tblOrder.id', ondelete='RESTRICT'), nullable=False),
        Column('payment_method_id', Integer, ForeignKey('tblPaymentMethod.id', ondelete='RESTRICT'), nullable=True),
        Column('status', SmallInteger, nullable=False, default=order_models.OrderPaymentStatus.PAYMENT_PENDING),
        Column('description', UnicodeText, nullable=False),
        Column('payment_date', DateTime(), nullable=False),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
    )

    tables['tblCurrency'] = tblCurrency
    tables['tblCountryCurrency'] = tblCountryCurrency

    tables['tblTaxClass'] = tblTaxClass
    tables['tblTaxClassGroup'] = tblTaxClassGroup
    tables['tblTaxClassCountry'] = tblTaxClassCountry
    tables['tblTaxClassCountryRate'] = tblTaxClassCountryRate

    tables['tblGLAccount'] = tblGLAccount

    tables['tblInvoiceFile'] = tblInvoiceFile
    tables['tblNewInvoiceMail'] = tblNewInvoiceMail

    tables['tblPaymentMethod'] = tblPaymentMethod
    tables['tblPaymentMethodBankTransfer'] = tblPaymentMethodBankTransfer
    tables['tblPaymentMethodOgone'] = tblPaymentMethodOgone

    tables['tblOrder'] = tblOrder
    tables['tblOrderGroup'] = tblOrderGroup
    tables['tblOrderLine'] = tblOrderLine
    tables['tblOrderPaymentStatus'] = tblOrderPaymentStatus

    tables['aggOrder'] = aggOrder

    __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if __mapping_defined:
        return

    from basesoft.web.contrib.file import models as file_models
    from basesoft.web.contrib.file.sa_datamapper import get_tables as file_tables
    from basesoft.web.contrib.license.sa_datamapper import get_tables as license_tables
    from basesoft.web.contrib.mailing import models as mailing_models
    from basesoft.web.contrib.mailing.sa_datamapper import get_tables as mail_tables
    from basesoft.web.contrib.i18n import models as i18n_models

    mapper(order_models.Currency, tables['tblCurrency'],
        properties = {
        }
    )

    mapper(order_models.CountryCurrency, tables['tblCountryCurrency'],
        properties = {
            'country': relation(Country),
            'currency': relation(order_models.Currency),
        }
    )

    mapper(order_models.TaxClass, tables['tblTaxClass'],
        properties = {
            'groups': relation(order_models.TaxClassGroup, backref='tax_class', cascade="all", passive_deletes=True),
        }
    )

    mapper(order_models.TaxClassGroup, tables['tblTaxClassGroup'],
        properties = {
            'group': relation(license_models.Group)
        }
    )

    mapper(order_models.TaxClassCountry, tables['tblTaxClassCountry'],
        properties = {
            'country': relation(Country),
            'tax_class': relation(TaxClass),
            'rates': relation(order_models.TaxClassCountryRate, backref='country_tax', order_by=get_tables()['tblTaxClassCountryRate'].c.start_date, passive_deletes=True),
        }
    )

    mapper(order_models.TaxClassCountryRate, tables['tblTaxClassCountryRate'],
        properties = {
            'tax_class_country': relation(TaxClassCountry),
        }
    )

    mapper(order_models.GLAccount, tables['tblGLAccount'],
        properties = {
            'merchant': relation(license_models.License),
        }
    )


    mapper(order_models.InvoiceFile, tables['tblInvoiceFile'],
        polymorphic_identity='InvoiceFile',
        inherits=file_models.File,
        inherit_condition=file_tables()['tblFile'].c.id==tables['tblInvoiceFile'].c.file_id
    )

    mapper(order_models.NewInvoiceMail, tables['tblNewInvoiceMail'],
        extension = PrivilegeObjectMapper(),
        properties = {
            'fallback_locale': relation(i18n_models.Locale),
            'locales': relation(mailing_models.MailLocale, order_by=mail_tables()['tblMailLocale'].c.locale_id, passive_deletes=True)
        },
        polymorphic_identity='NewInvoiceMail',
        inherits=mailing_models.Mail
    )

    mapper(order_models.PaymentMethod, tables['tblPaymentMethod'],
        polymorphic_identity='AbstractPaymentMethod',
        polymorphic_on=tables['tblPaymentMethod'].c.polymorphic_type,
        with_polymorphic='*',
        properties = {
            'merchant': relation(license_models.License),
        }
    )

    mapper(order_models.PaymentMethodBankTransfer, tables['tblPaymentMethodBankTransfer'],
        polymorphic_identity='PaymentMethodBankTransfer',
        inherits=order_models.PaymentMethod
    )

    mapper(order_models.PaymentMethodOgone, tables['tblPaymentMethodOgone'],
        properties = {},
        polymorphic_identity='PaymentMethodOgone',
        inherits=order_models.PaymentMethod
    )


    mapper(order_models.Order, tables['tblOrder'],
        extension = PrivilegeObjectMapper(),
        properties = {
            'parent_order': relation(order_models.Order, uselist=False, primaryjoin=tables['tblOrder'].c.parent_order_id==tables['tblOrder'].c.id, remote_side=[tables['tblOrder'].c.id]),
            'groups': relation(order_models.OrderGroup, backref='order', cascade="all", passive_deletes=True),
            'invoice': relation(order_models.InvoiceFile),
            'lines': relation(order_models.OrderLine, backref='order', order_by=get_tables()['tblOrderLine'].c.index, cascade="all", passive_deletes=True),
            'payment_statuses': relation(order_models.OrderPaymentStatus, backref='order', order_by=get_tables()['tblOrderPaymentStatus'].c.last_updated),
            'client': relation(license_models.License,primaryjoin=tables['tblOrder'].c.client_id==license_tables()['tblLicense'].c.id),
            'merchant': relation(license_models.License,primaryjoin=tables['tblOrder'].c.merchant_id==license_tables()['tblLicense'].c.id),
            'currency': relation(order_models.Currency),
        },
        polymorphic_identity='Order',
        polymorphic_on=tables['tblOrder'].c.polymorphic_type,
    )

    mapper(order_models.OrderGroup, tables['tblOrderGroup'],
        properties = {
            'group': relation(license_models.Group)
        }
    )

    mapper(order_models.OrderLine, tables['tblOrderLine'],
        polymorphic_identity='OrderLine',
        polymorphic_on=tables['tblOrderLine'].c.polymorphic_type,
    )

    mapper(order_models.OrderPaymentStatus, tables['tblOrderPaymentStatus'],
        properties = {
            'payment_method': relation(order_models.PaymentMethod),
        }
    )

    __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Order')

def app_permissions():
    from django.utils.translation import ugettext as _
    return [

        license_models.Permission(u'manage_tax_classes', u"""
            This gives the right to manage tax classes
            """, _("Manage tax classes")),

        license_models.Permission(u'view_country_tax', u"""
            This gives the right to view Country tax settings
            """, _("View country tax")),

        license_models.Permission(u'manage_country_tax', u"""
            This gives the right to manage country taxes
            """, _("Manage country tax")),

        license_models.Permission(u'manage_gl_accounts', u"""
            This gives the right to manage GL Accounts
            """, _("Manage GL accounts")),

        license_models.Permission(u'view_orders', u"""
            This gives the right to view Orders, always required for any Order type.
            """, _("View orders")),

        license_models.Permission(u'pay_orders', u"""
            This gives the right to pay Orders, always required for any Order type.
            """, _("Pay orders")),

        license_models.Permission(u'complete_orders', u"""
            This gives the right to complete and reject Orders
            """, _("Complete orders")),

        license_models.Permission(u'remove_order_invoice', u"""
            This gives the right to remove an invoice from an order.
            """, _("Remove order invoice")),

        license_models.Permission(u'manage_payment_methods', u"""
            This gives the right to manage the available payment methods.
            """, _("Manage payment methods")),

        license_models.Permission(u'payment_method_bank_transfer', """
            This gives the right to use Bank Transfers as payment method
        """, _('Payment method: bank transfer')),
        license_models.Permission(u'payment_method_ogone', """
            This gives the right to use OGONE PSP as payment method
        """, _('Payment method: OGONE PSP')),

        license_models.Permission(u"create_invoice_mail", u"""
            This gives the right to create new invoice mails.
            """, _("Create invoice mails")),
        license_models.Permission(u"view_invoice_mail", u"""
            This gives the right to view invoice mails.
            """, _("View invoice mails")),
        license_models.Permission(u"update_invoice_mail", u"""
            This gives the right to update existing invoice mails.
            """, _("Update invoice mails")),
        license_models.Permission(u"delete_invoice_mail", u"""
            This gives the right to delete invoice mails.
            """, _("Delete invoice mails")),
    ]

def app_configs():
    from basesoft.web.contrib.account import models as account_models
    configs = []

    # The preferred payment method for the user account.
    config = account_models.Config()
    config.name = 'basesoft.web.contrib.order.preferred_payment_method'
    config.type = 'integer'
    config.default = order_models.Order.METHOD_BANK_TRANSFER
    config.description = u'Preferred payment method.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.contrib.order.receive_invoice_email'
    config.type = 'bool'
    config.default = True
    config.description = u'Receive invoice emails.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.contrib.order.minimum_invoiceable_amount'
    config.type = 'integer'
    config.default = 0
    config.description = u'Minimum invoiceable amount.'
    configs.append(config)

    return configs
