'''
Created on Oct 30, 2013

@author: marten
'''
from django.conf import settings

from basesoft.web.core.util.controllers import LicenseConfigController
from basesoft.web.contrib.mailing.schedule import HtmlMailTask
from basesoft.web.contrib.order.models import Order

from basesoft import logging
from iqportal.iqube.core.schedule import AbstractAggregateTask
from basesoft.web.contrib.order.controllers import OrderAggregator
logger = logging.getLogger('basesoft.web.contrib.order.schedule')

class NewInvoiceMailTask(HtmlMailTask):
    INTERVAL = 600 # every 10 minutes

    def execute(self):
        """
        """
        orders = self._get_orders_with_new_invoices()

        # update invoice stage
        for_sending = []
        for order in orders:
            lcc = LicenseConfigController(self.sa_session, scope=order.client)
            if not lcc.get('basesoft.web.contrib.order.receive_invoice_email'):
                self.logger.info('%s: client (%s) disabled receiving of invoice emails.' % (self, order.client.id))
                order.invoice_stage = Order.INVOICE_STAGE_SKIPPED
            else:
                order.invoice_stage = Order.INVOICE_STAGE_SENDING
                for_sending.append(order)
            self.sa_session.add(order)

        self.sa_session.flush()

        for order in for_sending:
            mail = self._get_mail(order)
            if mail is None:
                self.logger.warn('%s: no invoice mail available for client (%s), aborting' % (self, order.client.id))
                continue

            person = order.client.company.invoice_person
            if person is None:
                person = order.client.company.contact_person
            if person is None:
                self.logger.warn('%s: no person found to deliver invoice email for client (%s), aborting', (self, order.client.id))
                continue

            msg, result = self._build_message(person, mail, context={
                'order': order
            }, mail_from=mail.mail_from(), postfix_subject='#%s' % order.id, attachments=[order.invoice])

            self._send_mail(person.email, msg, result)

            # change invoice stage to "sent"
            order.invoice_stage = Order.INVOICE_STAGE_SENT
            self.sa_session.add(order)
            self.sa_session.flush()

            logger.info('%s: new invoice mail sent to client (%s)' % (self, order.client.id) )

    def _get_orders_with_new_invoices(self):
        return self.sa_session.query(Order).filter(Order.invoice_stage==Order.INVOICE_STAGE_NEW).all()

    def _get_mail(self, order):
        """
        return the new invoice mail object
        """
        oc = order.get_order_controller()(self.sa_session)
        return oc.get_new_invoice_mail(order)

    def __repr__(self):
        return '<NewInvoiceMailTask>'

class AggregateOrderTask(AbstractAggregateTask):
    refresh_datasources = ('%s.Order' % settings.SETTINGS_MODULE, )

    def populate(self):
        affected_rows = 0

        c = OrderAggregator()
        self._aggregators.append(c)
        affected_rows = affected_rows + c.run(loop_validator=self.loop_validator)

        return affected_rows

# tasks so the scheduler can pick them up
tasks = (NewInvoiceMailTask, AggregateOrderTask, )

