'''
Created on Dec 11, 2013

@author: martijn
'''
from django.http import HttpResponse
from basesoft.web.contrib.order.controllers import OgonePaymentController
from basesoft.web.contrib.order.models import PaymentMethodOgone
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.utils.translation import ugettext as _
from django.utils import simplejson

#
# Ogone views
#
# The views below are views to where the user is redirect after the payment process
# Dont do business logic here, they are just to inform the user.
#
#

def _response(request, status, params):
    result = {
        'params': params,
        'status': status
    }
    result_str = simplejson.dumps(result)

    return render_to_response('order/ogone/payment_finish.html', {
        'result': result_str,
    }, context_instance=RequestContext(request) )

def payment_method_ogone_accept(request, psp_id):
    return _response(request, 'accept', request.GET)

def payment_method_ogone_decline(request, psp_id):
    return _response(request, 'decline', request.GET)

def payment_method_ogone_exception(request, psp_id):
    return _response(request, 'exception', request.GET)

def payment_method_ogone_cancel(request, psp_id):
    return _response(request, 'cancel', request.GET)

#
# ogone direct request
#
# the views below are called directly from the ogone server to iqportal
#

def _get_controller(request, psp_id):
    sa_session = request.Session()
    c = OgonePaymentController(sa_session, psp_id, request=request)
    return c


def payment_method_ogone_direct_accept(request, psp_id):
    """

    This view is called directly by the ogone server after the payment is 'accepted', 'onhold' or 'uncertain'.
    At this point the user is still on the ogone page and is about to be redirected to payment_method_ogone_* view

    Notice: 'payment refuned' does not seem give a callback. Instead the user is asked to try again

    request.POST with status 'accept':
    <QueryDict: {u'orderID': [u'29'], u'STATUS': [u'9'], u'IP': [u'81.39.182.3'], u'PAYID': [u'26077562'], u'CN': [u'InvoiceFirstName InvoiceLastName'], u'NCERROR': [u'0'], u'TRXDATE': [u'12/13/13'], u'CARDNO': [u'XXXXXXXXXXXX1111'], u
    'BRAND': [u'VISA'], u'ACCEPTANCE': [u'test123'], u'currency': [u'EUR'], u'amount': [u'453.75'], u'SHASIGN': [u'E33D92934D2289F964F557B9EE2054573D040D51'], u'ED': [u'0220'], u'PM': [u'CreditCard']}

    request.POST with status 'ucertain':
    <QueryDict: {u'orderID': [u'32'], u'STATUS': [u'92'], u'IP': [u'81.39.182.3'], u'PAYID': [u'26080461'], u'CN': [u'InvoiceFirstName InvoiceLastName'], u'NCERROR': [u'20001001'], u'TRXDA
    TE': [u'12/13/13'], u'CARDNO': [u'XXXXXXXXXXXX6666'], u'BRAND': [u'VISA'], u'ACCEPTANCE': [u''], u'currency': [u'EUR'], u'amount': [u'453.75'], u'SHASIGN': [u'A69932DBB05647DC7F521ED8A5291F1561A9BAB8'], u'ED': [u'0122'], u'PM': [u'
    CreditCard']}>
    """

    _get_controller(request, psp_id).payment_process(request.POST)
    return HttpResponse('OK')

def payment_method_ogone_direct_cancel(request, psp_id):
    """
    This view is called "If the payment's status is "cancelled by the client" or "too many rejections by the acquirer".
    """
    _get_controller(request, psp_id).payment_cancel(request.POST)
    return HttpResponse('OK')

def payment_method_ogone_direct_statuschange(request, psp_id):
    """
    FIXME: implement when we know the POST contents
    """

    # prints below are needed to implement this
    print 'payment_method_ogone_direct_statuschange()'
    print request.POST

    _get_controller(request, psp_id).payment_process(request.POST)
    return HttpResponse('OK')

