'''
Created on Aug 26, 2014

@author: marten
'''
from django.conf import settings

from basesoft.scheduler.task import SIGNAL_STOP

from basesoft.web.contrib.account.decorators import login_required
from basesoft.web.core.util.controllers import PermissionController
from basesoft.web.core.exceptions import PublicSystemException
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher

@login_required
def widget_scheduler_server(request):
    """
    Widget view for communicating to the scheduler server.
    """

    import xmlrpclib

    pc = PermissionController(request.Session(), account_id=request.session['user_id'])
    pc.assert_permissions('scheduler_manager')

    def client():
        try:
            client = xmlrpclib.Server(settings.SCHEDULER_SERVER_XMLRPC, use_datetime=True, allow_none=True)
            client.echo('test') # issue a 'test' call that thrown an exception if the server is down
            return client
        except Exception, e:
            raise PublicSystemException('Unable to connect to scheduler server at %s: %s' % (settings.SCHEDULER_SERVER_XMLRPC, e.message))

    class rpc:

        def list_registered_tasks(self):
            return client().list_registered_tasks()

        def get_task_results(self, task_id):
            return client().get_task_results(int(task_id))

        def get_task_results_by_name(self, names):
            return client().get_task_results_by_name(names)

        def start_task(self, task_id):
            return client().start_task(int(task_id), None)

        def stop_process(self, process_id):
            return client().send_process_signal(int(process_id), SIGNAL_STOP)

        def enable_task(self, task_id, enable):
            return client().enable_task(int(task_id), enable)

        def list_processes(self):
            return client().list_processes()

        def disable_all_tasks(self):
            for task in client().list_registered_tasks():
                self.enable_task(task['task_id'], False)

        def enable_all_tasks(self):
            for task in client().list_registered_tasks():
                self.enable_task(task['task_id'], True)


    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(rpc())
    return dispatcher.handle_request(request)