# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

import time

from django.utils.translation import ugettext as _

from django.conf import settings
from django.utils.cache import patch_vary_headers
from django.utils.http import cookie_date

#this is what we are replacing
#from django.contrib.sessions.models import Session
from django.contrib.sessions.backends.base import SessionBase
from django.core.exceptions import SuspiciousOperation
import datetime

from sqlalchemy.exc import OperationalError, InvalidRequestError
from sqlalchemy import and_

# this sets up the models and mappings
from basesoft.web.contrib.session import models
from basesoft.web.core.exceptions import LoginRequiredException,\
    PublicUserException
from sqlalchemy import create_engine
from basesoft.web.core.util.db import Session


class SASessionMiddleware(object):
    """
    This middleware is based on the exisiting django session
    middleware, but implements db storage with sqlalchemy
    """
    def process_request(self, request):
        """
        Make the session dict available in request object
        """
        session_key = request.COOKIES.get(settings.SESSION_COOKIE_NAME, None)

        # use redis or db session storage
        if settings.REDIS_HOST is not None:
            from basesoft.web.contrib.session.engines.redis_store import SessionStore as RedisSessionStore
            request.session = RedisSessionStore(session_key)
        else:
            from basesoft.web.contrib.session.engines.db_store import SessionStore as DbSessionStore
            request.session = DbSessionStore(session_key)

        request.META['IQPORTAL_VERSION'] = settings.VERSION

    def process_response(self, request, response):
        """
        If request.session was modified, or if response.session was set, save
        those changes and set a session cookie.
        """
        try:
            accessed = request.session.accessed
            modified = request.session.modified
        except AttributeError:
            pass
        else:
            if accessed:
                patch_vary_headers(response, ('Cookie',))
            if modified:
                if settings.SESSION_EXPIRE_AT_BROWSER_CLOSE:
                    max_age = None
                    expires = None
                else:
                    if (request.session.has_key('keep_logged_in') and request.session['keep_logged_in']) or \
                        (request.session.has_key('impersonator_keep_logged_in') and request.session['impersonator_keep_logged_in']):
                        max_age = settings.KEEP_SIGNED_COOKIE_AGE
                    else:
                        max_age = settings.SESSION_COOKIE_AGE
                    expires_time = time.time() + max_age
                    expires = cookie_date(expires_time)
                # Save the seesion data and refresh the client cookie.
                request.session.save()
                response.set_cookie(settings.SESSION_COOKIE_NAME,
                        request.session.session_key, max_age=max_age,
                        expires=expires, domain=settings.SESSION_COOKIE_DOMAIN,
                        path=settings.SESSION_COOKIE_PATH,
                        secure=settings.SESSION_COOKIE_SECURE or None)

        # close the database for connection pooling to happen
        request.Session().close()

        return response
