# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class TicketManagerIcon(IconBarIcon):
    """
    Class responsible for handling the ticket manager icon
    """

    id = 'icon-ticket-manager'
    title = _('Ticket manager')
    href = '#' if settings.TRAC_EXTERNAL_URL is None else '%s%s' % (settings.TRAC_EXTERNAL_URL, '#m=ticket-overview')
    target = '_blank'
    icon_index = 91
    group = 'ticketing'

    char_main = u'\uE824' # Clipboard icon
    char_sub = u'\uE810' # File icon

    # show icon with number of new changes to the tickets
    number_type = 'important'
    number_rpc_url = None if settings.TRAC_DB_CONNSTR is None else 'basesoft.web.contrib.ticketing.TicketRPCHandler'
    number_rpc_method = 'get_icon_count'
    number_refresh = 900 # refresh 15 minutes
    number_rpc_login_required = True # only refreshes if user is logged

    def _get_component(self):
        if settings.TRAC_EXTERNAL_URL is not None:
            return None
        return 'iq_ticket_manager' if settings.TRAC_DB_CONNSTR is None else 'iq_custom_ticket_manager'

    component = property(_get_component)

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('view_ticket')

class NewTicketIcon(IconBarIcon):
    """
    Class responsible for handling the ticket panel icon
    """

    id = 'icon-new-ticket'
    title = _('New ticket')
    href = '#' if settings.TRAC_EXTERNAL_URL is None else '%s%s-%s-%s' % (settings.TRAC_EXTERNAL_URL, '#m=ticket-create', settings.TRAC_IQPORTAL_VERSION, settings.VERSION)
    target = '_blank'
    icon_index = 92
    component = 'iq_ticket_panel'
    group = 'ticketing'

    char_main = u'\uE810' # File icon
    char_sub = u'\uE816' # Plus icon

    def _get_component(self):
        if settings.TRAC_EXTERNAL_URL is not None:
            return None
        return 'iq_ticket_panel'

    component = property(_get_component)

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('create_ticket')

class TicketAdminIcon(IconBarIcon):
    """
    Class responsible for handling the ticket admin icon
    """

    id = 'icon-ticket-admin'
    title = _('Ticket admin')
    href = '#'
    icon_index = 93
    component = 'iq_ticket_admin'
    group = 'ticketing'

    char_main = u'\uE810' # File icon
    char_sub = u'\uE823' # Lines icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('ticket_admin')

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [TicketManagerIcon(), NewTicketIcon(), TicketAdminIcon()]
