# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.ticketing.controllers import TicketController

class TicketRPCHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.controller = TicketController(request.Session(), account_id=request.session['user_id'])

    def load_ticket_state(self, ticket_id, is_trac_id=False):
        """
        Return the state of the ticket
        """
        return self.controller.load_ticket_state(ticket_id, is_trac_id=is_trac_id)

    def get_ticket_types(self):
        """"
        Return a list of ticket types that's used for the values of the ticket type dropdown
        """
        ticket_types = [{'value': t, 'label': t, 'default': False} for t in self.controller.get_ticket_types()]
        if len(ticket_types) > 0:
            ticket_types[0]['default'] = True
        return ticket_types

    def save_ticket(self, state):
        """
        Save the ticket using the state and return the state of the saved/updated ticket
        """
        return self.controller.save_or_update_ticket(state)

    def get_ticket_info(self):
        """
        Return the summaries of the tickets for the current user
        """
        return self.controller.get_ticket_info()

    def get_non_privileged_tickets(self):
        """
        Return a list of tickets that are not yet in IQPortal as privileged object.
        """
        return self.controller.get_non_privileged_tickets()

    def load_non_privileged_ticket(self, id):
        return self.controller.load_ticket_state(id, True)

    def save_ticket_to_privileged_object(self, values):
        self.controller.save_ticket_to_privileged_object(**values)
        return True

    def get_icon_count(self):
        return self.controller.get_new_count()

handlers = (TicketRPCHandler, )


