# Copyright (c) 2015 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from basesoft.util.virtual_date import today, set_date
from basesoft.web.core.exceptions import PublicSystemException
from basesoft.web.core.util.controllers import AbstractController

class UATHandler:

    def __init__(self, request, **kwargs):

        self._request = request
        self._sa_session = request.Session()
        self._account_id = request.session['user_id']

        self._controller = AbstractController(self._sa_session, account_id=self._account_id)

    def _get_client(self):
        import xmlrpclib

        self._controller.assert_permissions('user_acceptance_trigger_tasks')
        try:
            client = xmlrpclib.Server(settings.SCHEDULER_SERVER_XMLRPC, use_datetime=True, allow_none=True)
            client.echo('test') # issue a 'test' call that thrown an exception if the server is down
            return client
        except Exception, e:
            raise PublicSystemException('Unable to connect to scheduler server at %s: %s' % (settings.SCHEDULER_SERVER_XMLRPC, e.message))

    def get_virtual_date(self):
        return today()

    def set_virtual_date(self, dt):
        self._controller.assert_permissions('set_virtual_date')
        set_date(dt)
        return True

    def get_date_and_task_status(self, tasks=None):

        t = []
        if tasks is not None:
            client = self._get_client()
            registered = client.list_registered_tasks(True)
            [t.append(i) for i in registered if i['name'] in tasks]

        return {
            'virtual_date': today(),
            'tasks': t
        }

    def trigger_task(self, task_id):
        client = self._get_client()
        client.start_task(int(task_id), None)
        return True

    def get_task_status(self, tasks):
        client = self._get_client()
        registered = client.list_registered_tasks(True)
        return [i for i in registered if i['name'] in tasks]

handlers = (UATHandler,)
