# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

import re, cgi
from basesoft.web.contrib.wiki.formatter import HTMLFormatter

class Macro:
    """
    """
    def __init__(self, db=None, session=None):
        """
        """
        self.db = db
        self.session = session

class QuoteBlock(Macro):
    """
    """
    options = re.compile('^(?:"(?P<quote>[^"]*)","(?P<quotee>[^"]*)"(?:,"(?P<link>.*)")?)|(?P<fullquote>(.*))$');


    def execute(self, quote):
        """
        """
        m = self.options.match(quote)
        if m.group('quote') is None:
            return '<div class="quote"><span class="quote">"%s"</span></div>' % m.group('fullquote')

        legend = m.group('quotee')
        if m.group('link') is not None:
            legend = '<a href="%s" target="_blank">%s</a>' % (m.group('link'), legend)

        return '<div class="quote"><span class="quote">"%s"</span> - %s</div>' % (m.group('quote'), legend)

class Image(Macro):
    def execute(self, image_hash):
        return '<img src="/basesoft.web.contrib.file/view/%s" />' % image_hash

class ExternalImage(Macro):
    def execute(self, image_location):
        return '<img src="%s" />' % image_location

class Footer(Macro):
    def execute(self, footer_text):
        return '<div class="page-footer">%s</div>' % footer_text

class BR(Macro):
    def execute(self, bitbucket):
        return '<br />'

class Parser:
    """
    """
    macros = {'QuoteBlock': QuoteBlock, 'Image': Image, 'Footer': Footer, 'ExternalImage': ExternalImage, 'BR': BR}

    def __init__(self):
        """
        """
        self.content = ''
        self.context = None

    def add_macro(self, name, obj):
        self.macros[name] = obj

    def set_content(self, content):
        """
        """
        self.content = content

    def set_context(self, context):
        self.context = context

    def render_to_html(self, content=None, context=None):
        """
        """
        result = ''
        if content is None:
            content = self.content
        if context is None:
            context = self.context

        try:
            f = HTMLFormatter(self.macros, context=context)
            result = f.format(content)
        except WikiMacroException, e:
            result = '<div class="wiki-error">%s</div>' % e.message

        return result

class WikiMacroException(Exception):
    """
    """

