# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

import re

class Provider:
    """
    """
    def __init__(self):
        """
        """
        self._prepare_rules()

class WikiProvider(Provider):
    """
    """
    _compiled_rules = False

    LOWER_THAN = "<"
    BOLDITALIC_TOKEN = "''''"
    BOLD_TOKEN = "'''"
    ITALIC_TOKEN = "''"
    UNDERLINE_TOKEN = "__"
    STRIKE_TOKEN = "~~"
    CENTER_TOKEN = "::"
    SUBSCRIPT_TOKEN = ",,"
    SUPERSCRIPT_TOKEN = r"\^"
    INLINE_TOKEN = "`"
    STARTBLOCK_TOKEN = r"\{\{\{"
    STARTBLOCK = "{{{"
    ENDBLOCK_TOKEN = r"\}\}\}"
    ENDBLOCK = "}}}"

    LINK_SCHEME = r"[\w.+-]+"  # as per RFC 2396

    QUOTED_STRING = r"'[^']+'|\"[^\"]+\""

    LHREF_RELATIVE_TARGET = r"[/#][^\s\]]*|\.\.?(?:[/#][^\s\]]*)?"

    XML_NAME = r"[\w:](?<!\d)[\w:.-]*?"

    # used for replacement fields
    OBJ_NAME = r"[\w_]+"
    PROPERTY_NAME = r"[\w_\.]+"

    PAGEBREAK = '--\{pagebreak\}--'

    _pre_rules = [
        r"(?P<lowerthan>!?%s)" % LOWER_THAN,
        r"(?P<bolditalic>!?%s)" % BOLDITALIC_TOKEN,
        r"(?P<bold>!?%s)" % BOLD_TOKEN,
        r"(?P<italic>!?%s)" % ITALIC_TOKEN,
        r"(?P<underline>!?%s)" % UNDERLINE_TOKEN,
        r"(?P<strike>!?%s)" % STRIKE_TOKEN,
        r"(?P<subscript>!?%s)" % SUBSCRIPT_TOKEN,
        r"(?P<superscript>!?%s)" % SUPERSCRIPT_TOKEN,
        r"(?P<pagebreak>!?%s)" % PAGEBREAK,
        r"(?P<inlinecode>!?%s(?P<inline>.*?)%s)" \
            % (STARTBLOCK_TOKEN, ENDBLOCK_TOKEN),
        r"(?P<inlinecode2>!?%s(?P<inline2>.*?)%s)" \
            % (INLINE_TOKEN, INLINE_TOKEN),
        r"(?P<heading>^\s*(?P<hdepth>=+)\s.*\s(?P=hdepth)\s*"
        r"(?P<hanchor>#%s)?(?:\s|$))" % XML_NAME,
        r"(?P<center>!?%s)" % CENTER_TOKEN,
        r'(?<=^|(?<=\s))(?:#)(?P<hashtag>[\w\d]+)?',
        (r"(?P<macro>!?\[\[(?P<macroname>[\w/+-]+)"
         r"(\]\]|\((?P<macroargs>.*?)\)\]\]))"),
        # [wiki:WikiLinks with optional label] or [/relative label]
        (r"(?P<lhref>!?\[(?:"
        r"(?P<rel>%s)|" % LHREF_RELATIVE_TARGET +  # ./... or /...
        r"(?P<lns>%s):(?P<ltgt>%s|[^\]\s]*))" % \
        (LINK_SCHEME, QUOTED_STRING) +  # wiki:TracLinks or wiki:"trac links"
        r"(?:\s+(?P<label>%s|[^\]]+))?\])" % QUOTED_STRING),  # optional label
        r"(?P<list>^(?P<ldepth>\s+)(?:[-*]|\d+\.|[a-zA-Z]\.|[ivxIVX]{1,5}\.) )",
        r"(?P<definition>^\s+((?:%s[^%s]*%s|%s.*?%s|[^%s%s:]|:[^:])+::)(?:\s+|$))"
            % (INLINE_TOKEN, INLINE_TOKEN, INLINE_TOKEN,
               STARTBLOCK_TOKEN, ENDBLOCK_TOKEN, INLINE_TOKEN, STARTBLOCK[0]),
        ]

    _replacement_rules = [
        r"(?<=^|(?<=[^a-zA-Z0-9-_\.]))(?:@)(?P<replacement>%s){1}(?:\:(?P<property>%s))?" % (OBJ_NAME, PROPERTY_NAME),
    ]

    _post_rules = [
        # || table ||
        r"(?P<last_table_cell>\|\|\s*$)",
        r"(?P<table_cell>\|\|)"]

    def _prepare_rules(self):
        if not self._compiled_rules:
            helpers = []
            handlers = {}
            syntax = self._pre_rules[:]
            syntax += self._post_rules[:]
            syntax += self._replacement_rules[:]
            helper_re = re.compile(r'\?P<([a-z\d_]+)>')
            for rule in syntax:
                helpers += helper_re.findall(rule)[1:]
            rules = re.compile('(?:' + '|'.join(syntax) + ')', re.UNICODE)
            self._external_handlers = handlers
            self._helper_patterns = helpers
            self._compiled_rules = rules

    def _get_rules(self):
        self._prepare_rules()
        return self._compiled_rules

    rules = property(_get_rules)

    def get_replacement_rules(self):
        return re.compile('(?:'+ '|'.join(self._replacement_rules)+')', re.UNICODE)
