# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

"""
This command is used to synchronized "default" data from the different apps (Permissions, Themes, etc)
"""
from twisted.python.reflect import namedAny

from django.core.management.base import BaseCommand
from django.conf import settings

from basesoft.web.core.util.db import Session

from basesoft import logging
from django.utils.translation import get_language
from sqlalchemy.sql.expression import text
logger = logging.getLogger("basesoft.django.commands.iqsyncdb")

class Command(BaseCommand):
    """
    """
    help = "Synchronizes default data for the supplied apps"
    args = "[appname...]"

    def handle(self, session=None, *app_names, **options):
        """
        """
        if len(app_names) == 0:
            app_names = settings.INSTALLED_APPS

        # check for current language.. we store some labels in the database (like configurable widgets)
        # that on runtime are passed into gettext.
        if get_language() != 'en':
            raise Exception('dbsync: language must be en')

        session = Session()

        for app in app_names:
            try:
                sync_module = namedAny(app+'.db_sync')
            except AttributeError:
                # if we do not have a sync module, we don't care for this app, continue...
                continue

            # call sync for current app
            if hasattr(sync_module, 'sync'):
                logger.info('Synchronizing database for app %s' % app)
                sync_module.sync(session)

            # execute procedures
            if hasattr(sync_module, 'procedures'):
                for s in sync_module.procedures():
                    session.execute(text(s))

            # call post sync on each app and pass in the current app name
            for post_sync_app in app_names:
                try:
                    post_sync_module = namedAny(post_sync_app+'.db_sync')
                except AttributeError:
                    # if we do not have a sync module, we don't care for this app, continue...
                    continue

                if not hasattr(post_sync_module, 'post_sync'):
                    continue

                #logger.info('post_sync for app %s, (%s as dependency)' % (post_sync_app, app))
                post_sync_module.post_sync(session, app)
