# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.core.management.base import BaseCommand
from django.conf import settings

from sqlalchemy import create_engine
from migrate.versioning.util import load_model

from migrate.versioning import genmodel, schemadiff

from basesoft import logging
logger = logging.getLogger("basesoft.django.commands.iqupgradedb")

class Command(BaseCommand):


    def handle(self, session=None, *app_names, **options):
        """
        """
        dryrun = options.get('dryrun', True)
        if dryrun:
            logger.info('Dryrun active')

        engine = create_engine(settings.DB_CONNSTR, echo=False)

        model = load_model('basesoft.web.core.util.db.metadata')

        diff = schemadiff.SchemaDiff(model, engine)
        if len(diff.tablesWithDiff)==0 and len(diff.tablesMissingInDatabase)==0 \
            and len(diff.tablesMissingInModel)==0 and len(diff.colDiffs)==0:
            logger.info('No schema diffs')
            return

        logger.info('Database out of sync')
        logger.info('Diff: %s' % diff)
        if not dryrun:
            logger.info('Applying changes')
            genmodel.ModelGenerator(diff, engine).applyModel()
        else:
            logger.info('Not applying changes, dryrun active')
