
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcResponseException
from django.http import HttpResponse, HttpResponseServerError
from basesoft.web.core.exceptions import PublicException
from django.shortcuts import render_to_response
from basesoft.logging import getLogger

class ExceptionMiddleware(object):
    """
    http://docs.djangoproject.com/en/dev/topics/http/middleware/#process-exception
    """

    def process_exception(self, request, exception):

        # check if the request is jsonrpc, then let the JsonRpcResponseException handle it
        if request.is_ajax():
            return JsonRpcResponseException(request, exception)

        # else if its a publc exception, just show it to the user
        elif isinstance(exception, PublicException):
            response = render_to_response('exception.html', {'exception_message': exception})
            response.status_code = 500
            return response

        elif isinstance(exception, IOError) and 'request data read error' in exception:
            logger = getLogger('ExceptionMiddleware', request)
            logger.info('%s %s: %s: Request was canceled by the client.' % (request.build_absolute_uri(), request.user, exception))
            return HttpResponseServerError()

        # other exceptions will be sent by django

