# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.core.exceptions import SecurityException, PublicSecurityException, IconException

class MobileIcon:
    """
    One Icon for the mobile app.
    """

    title = 'default title'
    desc = 'default description'
    image = None

    # the js file that contains the code responsible for the behaviour
    js_module = None

    # the index of the icon, used for ordering. lowest index appears first
    icon_index = 1 # default 1, because home is 0 and we want it first

    # when clicked 'app' tells the iq_bootloader to start that application.
    name = None

    # if shown in home menu
    in_home_menu = True

    def assert_permissions(self, *args):
        """
        This needs to be overwritten in children classes.
        """
        raise SecurityException('Icon doesn\'t have assert_permissions defined.')

    def has_permissions(self, controller):
        """
        Check if an icon has the needed permissions to be shown.

        @param request: the request that contains logged in users, etc.
        @return: bool True if icon has permissions, False if it doesn't.
        """
        try:
            self.assert_permissions(controller)
        except PublicSecurityException:
            return False
        except SecurityException:
            return False
        except IconException:
            return False
        return True

    def __str__(self):
        return '<MobileIcon "%s" >' % self.name

    def __cmp__(self, other):
        """
        Used for sorting the list of icons in the iconbar, based on icon_index
        Eg. sorted_icons = my_list_of_icons.sort()
        """
        return cmp(self.icon_index, other.icon_index)
