'''
Created on 16 Sep 2011

@author: martijn
'''

import time
from django.conf import settings

from django.utils import simplejson
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from basesoft.web.mobile.controllers import MobileController
from basesoft.web.core.views import loader_get
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder

if settings.DEBUG:
    manifest_revision = int(time.time())
else:
    manifest_revision = settings.VERSION

def mobile_home(request):

    account_id = None
    if request.session.has_key('user_id'):
        account_id = request.session['user_id']

    controller = MobileController(request.Session(), account_id)
    icons = controller.get_icons()
    icons_state = [];
    for icon in icons:
        if icon.in_home_menu:
            icon_state = {
                'name': icon.name,
                'title': icon.title,
                'desc': icon.desc,
                'image': icon.image,
            }
            icons_state.append(icon_state)

    return render_to_response('mobile_home.html', {'revision': int(time.time()), 'icons': simplejson.dumps(icons_state, ensure_ascii=False, cls=JsonRpcJSONEncoder) }, context_instance=RequestContext(request))

def cache_manifest(request):

    template_vars = {
        'manifest_revision': manifest_revision
    }

    response = render_to_response('cache.manifest', dictionary=template_vars, mimetype='text/cache-manifest')
    response['Cache-Control'] = 'no-cache'

    return response


def offline(request):
    return render_to_response('offline.html')

def bootstrap(request, version, type, extra_requires):

    # parse extra_requires
    if extra_requires == '':
        extra_requires = []
    else:
        extra_requires = extra_requires.split(',')

    # load requires based on mobile apps
    account_id = None
    if request.session.has_key('user_id'):
        account_id = request.session['user_id']
    controller = MobileController(request.Session(), account_id)
    icons = controller.get_icons()
    for icon in icons:
        if icon.js_module is not None:
            extra_requires.append(icon.js_module)

    # setup requires
    requires = [
        'iq_mobile_core',
    ] + extra_requires

    return loader_get(request, version, type, '', ','.join(requires))

