# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, ForeignKey, Unicode
from basesoft.sqlalchemy.table import IQTable
from django.utils.translation import ugettext as _
from basesoft.web.contrib.license import models as license_models

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:


        # FME Contact related tables
        tblFMEContact = IQTable('tblFMEContact', metadata,
            Column('contact_id', Integer, ForeignKey('tblContact.id', ondelete='CASCADE'), primary_key=True)
        )

        tblFMECompany = IQTable('tblFMECompany', metadata,
            Column('company_id', Integer, ForeignKey('tblCompany.id', ondelete='CASCADE'), primary_key=True),
            Column('company_number', Integer, nullable=False),
            Column('no_of_employees', Integer),
            Column('department_number', Unicode(150)),
            Column('busrelaccount', Unicode(150)),
            Column('CAO', Unicode(150))

        )

        tblFMEPerson = IQTable('tblFMEPerson', metadata,
            Column('person_id', Integer, ForeignKey('tblPerson.id', ondelete='CASCADE'), primary_key=True),
            Column('person_number', Integer, nullable=False),
            Column('internal_location', Unicode(150))
        )

        tables['tblFMEContact'] = tblFMEContact
        tables['tblFMECompany'] = tblFMECompany
        tables['tblFMEPerson'] = tblFMEPerson

        __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:
        from basesoft.web.contrib.contact import models as contact_models
        from iqportal.client.core import models as client_models

        mapper(client_models.FMEContact, tables['tblFMEContact'],
            polymorphic_identity='FMEContact',
            inherits=contact_models.Contact
        )

        mapper(client_models.FMECompany, tables['tblFMECompany'],
            polymorphic_identity='FMECompany',
            inherits=contact_models.Company
        )

        mapper(client_models.FMEPerson, tables['tblFMEPerson'],
            polymorphic_identity='FMEPerson',
            inherits=contact_models.Person
        )

        __mapping_defined = True

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Client specific')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the `iqsyncpermissions` command.

    Titles are with gettext calls, but in the database they appear in English. We need it like this so we get the
    titles in the language files. Later, in the js interfaces we show them translated.
    """
    return [
        license_models.Permission(u'fme_contact_access', u"""
            This gives the right to FME specific contact manager.
            """, _("FME Contact manager")),
        ]
