# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from types import NoneType

from django.conf import settings
from django.utils import simplejson as json
from django.template.loader import render_to_string
from django.template import RequestContext

from basesoft.util.image import direct_image
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder

from basesoft import logging
from basesoft.web.contrib.graph.controllers import GraphController
from basesoft.web.core.util.db import Session
logger = logging.getLogger("iqreport.test.yui")

default_conf = {
    # list of dictionaries that have chart type ['bar', 'line'] as key and list of dimensions as values
    # eg. [{'bar': ['Dissatisfied']}, {'line': ['NCS Score']}]
    # anything not found in the dictionary keys is part of the base chart, the rest are appended to that in the
    # given order. If splity_by is an empty list, the chart is just a simple one, no combinations
    'split_by': (list, [], None),

    'title': ((str, unicode), '', None),
    'show_title': (bool, True, None),

    'type': ((str, unicode), 'stcolumn3d', (

        # single
        'column2d', 'column3d', 'pie2d', 'pie3d', 'line2d', 'bar2d', 'area2d', 'doughnut2d', 'doughnut3d', 'pareto2d', 'pareto3d',
        # multi
        'msline2d', 'mscolumn2d', 'mscolumn3d', 'msbar2d', 'msbar3d', 'msline2d', 'mszoomline', 'msarea2d', 'msmarmekko',
        # stacked
        'stcolumn2d', 'stcolumn3d', 'stbar2d', 'stbar3d', 'starea2d',

        # combination
        'stcolumn3dline',
    )),

    'aspect_ratio': ((int, float), 1.77, None),
    'aspect_ratio_custom': ((int, float), 1, None),

    # fixme: what is still used from this
    'color_theme': ((str,unicode), 'default', ('default', 'contrast', 'light')),
    'color_set': ((str,unicode), '', None),
    'graph_theme': ((str,unicode), '', None),
    'override_colors': ((str,unicode,NoneType), None, None),

    # common
    'graphs_per_line': (int, 3, None),
    'limit_nr_graphs': (int, None, None),

    #
    # fusion charts specific
    #

    # functional
    'fusion_chart.showLabels':  (bool, True, None),
    'fusion_chart.showValues':  (bool, True, None),
    'fusion_chart.showPercentValues':  (bool, False, None),

    # legend
    'fusion_chart.showLegend':  (bool, True, None),
    'fusion_chart.interactiveLegend':  (bool, True, None),

    'fusion_chart.legendPosition':  ((str, unicode), 'RIGHT', ('BOTTOM', 'RIGHT')),
    'fusion_chart.legendCaption':  ((str, unicode, NoneType), None, None),
    'fusion_chart.legendIconScale':  ((int, NoneType), None, None),
    'fusion_chart.legendNumColumns':  ((int, NoneType), None, None),
    'fusion_chart.minimiseWrappingInLegend':  (bool, False, None),

    # margin/padding
    'fusion_chart.captionPadding':  ((int, NoneType), None, None),
    'fusion_chart.canvasPadding':  ((int, NoneType), 10, None),
    'fusion_chart.chartLeftMargin':  ((int, NoneType), 1, None),
    'fusion_chart.chartRightMargin':  ((int, NoneType), 1, None),
    'fusion_chart.chartTopMargin':  ((int, NoneType), 1, None),
    'fusion_chart.chartBottomMargin':  ((int, NoneType), 1, None),

    # line
    'fusion_chart.lineThickness':  ((int, NoneType), None, None),
    'fusion_chart.lineDashed':  (bool, False, None),
    'fusion_chart.lineDashLen':  ((int, NoneType), None, None),
    'fusion_chart.lineDashGap':  ((int, NoneType), None, None),

    #'fusion_chart.lineColor': ((str, unicode), '#000', None),

    'fusion_chart.drawAnchors':  (bool, True, None),
    'fusion_chart.anchorSides':  ((int, NoneType), None, None),
    'fusion_chart.anchorRadius':  ((int, NoneType), None, None),


    # pie/donut specific
    'fusion_chart.showZeroPies':  (bool, True, None),
    'fusion_chart.showPercentInToolTip':  (bool, True, None),
    'fusion_chart.pieRadius':  ((int, NoneType), None, None),
    'fusion_chart.startingAngle':  (int, 90, None),
    'fusion_chart.pieYScale':  (int, 60, None),
    'fusion_chart.pieSliceDepth':  ((int, NoneType), None, None),
    'fusion_chart.smartLabelClearance':  ((int, NoneType), None, None),
    'fusion_chart.manageLabelOverflow':  (int, True, None),

    # column/bar 3d specific
    'fusion_chart.maxColWidth':  ((int, NoneType), None, None),
    'fusion_chart.maxColHeight':  ((int, NoneType), None, None),

    # combo charts
    'render_line_indexes':  ((str, unicode, NoneType), None, None),


    # not pie specific
    'fusion_chart.labelDisplay':  ((str, unicode), 'AUTO', ('AUTO', 'WRAP', 'STAGGER', 'ROTATE', 'NONE')),
    'fusion_chart.slantLabels':  (bool, False, None),
    'fusion_chart.staggerLines':  (int, 2, None),
    'fusion_chart.rotateValues':  (bool, False, None),
    'fusion_chart.placeValuesInside':  (bool, False, None),
    'fusion_chart.yAxisMinValue':  ((int, NoneType), None, None),
    'fusion_chart.yAxisMaxValue':  ((int, NoneType), None, None),
    'fusion_chart.setAdaptiveYMin':  (bool, False, None),

    'fusion_chart.xAxisName': ((str, unicode, NoneType), None, None),
    'fusion_chart.yAxisName': ((str, unicode, NoneType), None, None),


    # trend lines
    'fs_tl0_startValue': ((int, NoneType), None, None),
    'fs_tl0_color': ((str, unicode, NoneType), '#009933', None),
    'fs_tl0_displayvalue': ((str, unicode, NoneType), None, None),

    # eye candy
    'fusion_chart.showShadow':  (bool, False, None),
    'fs_shadow_enable':  (bool, False, None),
    'fs_shadow_distance':  (int, 10, None),
    'fs_shadow_angle':  (int, 45, None),
    'fs_shadow_color': ((str, unicode), '#cccccc', None),

    'fs_glow_enable':  (bool, False, None),
    'fs_glow_color': ((str, unicode), '#FF0000', None),
    'fs_glow_alpha':  (int, 25, None),
    'fs_glow_strength':  (int, 2, None),

    'fs_bevel_enable':  (bool, False, None),
    'fs_bevel_angle':  (int, 45, None),
    'fs_bevel_distance':  (int, 4, None),
    'fs_bevel_shadow_color': ((str, unicode), '#000000', None),
    'fs_bevel_shadow_alpha': (int, 25, None),
    'fs_bevel_hilight_color': ((str, unicode), '#FFFFFF', None),
    'fs_bevel_hilight_alpha': (int, 25, None),
    'fs_bevel_strength':  (int, 4, None),

}


def js_module_iq_report_env(request):
    """
    This returns some enviroment data we need in a dictionary.
    This is referenced by the iq_report_env module in js_modules
    """

    # config default
    defaults = {}
    for key, value in default_conf.items():
        defaults[key] = value[1] # 0 is the type, 1 default value, 2 possible values
    config_defaults = json.dumps(defaults, ensure_ascii=False, cls=JsonRpcJSONEncoder)

    # colors
    colorsets = {}
    user_id = None
    if request.session.has_key('user_id'):
        user_id = request.session['user_id']
    gc = GraphController(Session(), account_id=user_id, request=request)
    for cs in gc.get_color_sets():
        colors = json.loads(cs.color_config)

        # repeat colors till 1000 so we dont get indexes errors
        for i in range( 1000-len(colors) ):
            colors.append(colors[i % len(colors) ]);

        colorsets[cs.name] = colors
    colorsets = json.dumps(colorsets, ensure_ascii=False, cls=JsonRpcJSONEncoder)

    return render_to_string("iqreport_env.html", {
        "config_defaults": config_defaults,
        "colorsets": colorsets,
    }, context_instance=RequestContext(request))

def direct_graph(request, hash):
    """
    No login is required for a direct link to a graph like this.
    This will return the image that can be used as src
    """
    return direct_image("%s%s.png" % (settings.IQREPORT_GRAPH_LOCATION, hash))

def mdx_names_form_rpc(request):
    """
    """
    from basesoft.web.core.forms.handler import FormValidator
    from basesoft.web.core.input.validation import plugins
    form_handler = FormValidator({
        'mdx_name': [(plugins.Required, {})],
        'mdx_result_name': [(plugins.Required, {})]
    })
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)
