# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

import datetime
from sqlalchemy import Column, Integer, ForeignKey, String, DateTime, Unicode, UnicodeText, SmallInteger

from basesoft.sqlalchemy.table import IQTable

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables


def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblReportViewer = IQTable('tblReportViewer', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('invitee_id', Integer, ForeignKey('tblComposeInvitee.id', ondelete="CASCADE")),

            Column('number_of_views', Integer, default=0, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblReportViewer'] = tblReportViewer

        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:
        from sqlalchemy.orm import mapper, relation
        from basesoft.web.contrib.compose import models as compose_models
        from iqportal.iqreport.display import models as display_models

        mapper(display_models.ReportViewer, tables['tblReportViewer'],
            properties = {
                'invitee': relation(compose_models.ComposeInvitee),
        })

        __mapping_defined = True


def fill_data(conn):
    pass
