# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.template import TemplateDoesNotExist

from basesoft.web.core.exceptions import PublicUserException
from basesoft.web.contrib.i18n import models as i18n_models

from iqportal.iqreport.display.controllers import ReportDisplayController
from basesoft.web.contrib.compose.views import compose_runner
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.contrib.branding.controllers import LicenseThemeController


def bootstrap(request, version, type, extra_requires):
    requires = ['iq_core', 'iq_loader']

    # parse extra_requires
    if extra_requires == '':
        extra_requires = []
    else:
        extra_requires = extra_requires.split(',')
    requires.extend(extra_requires)
    from basesoft.web.core.views import loader_get

    return loader_get(request, version, type, '', ','.join(requires))

def display(request, hash, viewer_print=None):
    """
    """

    print_ = 1 if viewer_print else 0

    sa_session = request.Session()


    user_id = None
    if request.session.has_key('user_id'):
        user_id = request.session['user_id']

    ltc = LicenseThemeController(sa_session, account_id=user_id, request=request)
    current_theme = ltc.get_current_theme()
    theme_css = current_theme.css_class
    theme_module_name = current_theme.module_name
    request.session['theme_module_name'] = theme_module_name


    report_viewer_id = None
    if request.session.has_key('report_publication_id'):
        del request.session['report_publication_id']
    if request.session.has_key('report_viewer_id'):
        report_viewer_id = request.session['report_viewer_id']

    # FIXME: locale from session #3238
    locale = sa_session.query(i18n_models.Locale).filter_by(locale='en_GB').one()

    lc = LicenseController(request.Session())
    site_licensee = lc.get_license_for_request(request)
    licensee = site_licensee.http_host

    # try to init report display controller, fallback to login dialog
    try:
        pc = ReportDisplayController(sa_session, locale, hash, report_viewer_id)
        request.session['report_viewer_id'] = pc.get_report_viewer().id
        assert request.session['report_viewer_id'] is not None

    except PublicUserException, e:
        try:
            return render_to_response('licensee/%s_login.html' % licensee, {'error': e.message}, context_instance=RequestContext(request))
        except TemplateDoesNotExist:
            return render_to_response('default_login.html', {'error': e.message}, context_instance=RequestContext(request))

    publication = pc.get_publication()
    # render report
    _vars = {
        'print': print_,
        'invitee_hash': hash,
        'report_viewer_id': pc.get_report_viewer().id,
        'compose_name': publication.report.name,
        'publication_label': publication.label,
        'show_compose_label': publication.show_compose_label,
        'show_publication_label': publication.show_publication_label,
        'theme_css': theme_css
    }

    title = 'IQ Report - %s - %s' % (pc.get_publication().report.name, pc.get_publication().label)

    return compose_runner(request, ['iqreport_init'], _vars, title, licensee)
