# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class IQubeMdxCalculatedMemberDesigner(IconBarIcon):
    id = 'mdx-calculated-member-designer'
    title = _('Mdx calculated member designer')
    icon_index = 49
    app = 'mdx'
    group = 'iqube'

    char_main = u'\uE85B' # MDX icon
    char_sub = u'\uE811' # Pencil icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        # 'use_raw_mdx', is temporary so clients dont get access to the Calculated Members yet.
        controller.assert_permissions('use_raw_mdx', 'create_calculated_member')

class IQubeMdxFormulaWidgetManager(IconBarIcon):
    id = 'mdx-formula-widget-manager'
    title = _('Mdx formula overview')
    icon_index = 50
    component = 'iqube_mdx_calculated_member_manager'
    group = 'iqube'

    char_main = u'\uE85B' # MDX icon
    char_sub = u'\uE82B' # List icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        # 'use_raw_mdx', is temporary so clients dont get access to the Calculated Members yet.
        controller.assert_permissions('use_raw_mdx', 'view_calculated_member')

class IQubeMdxFormatStringManager(IconBarIcon):
    id = 'mdx-format-string-manager'
    title = _('Format string overview')
    icon_index = 51
    component = 'iqube_format_string_manager'
    group = 'iqube'

    char_main = u'\uE85B' # MDX icon
    char_sub = u'FS' # FIXME

    @assert_user_logged_in
    def assert_permissions(self, controller):
        controller.assert_permissions('manage_format_strings')

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [IQubeMdxCalculatedMemberDesigner(), IQubeMdxFormulaWidgetManager(), IQubeMdxFormatStringManager()]
