# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _
from basesoft.util.ordered_dict import OrderedDict
from iqportal.iqube.core.controllers import ConfigurableWidgetController, ConfigurableWidgetStateController
from iqportal.iqube.core.models import DataSourceWidget

class ConfigurableWidget(ConfigurableWidgetStateController):
    name = 'iqube_db_configurable_widget'
    icon = '16x16/mimetypes/x-office-presentation.png'
    label = _('Configurable widget')
    category = _('Report Widgets')
    hidden = True
    permissions = ['view_datasource',]

class DbConfigurableWidget(ConfigurableWidget):
    hidden = False

widgets_mapper = {'DbConfigurableWidget': ConfigurableWidget}

def get_dynamic_widgets(session, account_id):
    widgets = OrderedDict()
    cwc = ConfigurableWidgetController(session, account_id=account_id)
    if cwc.has_permissions('view_datasource_widget'):
        for widget in cwc.get_configurable_widgets(availability=DataSourceWidget.AVAILABLE_AS_DASHBOARD_WIDGET):
            # make a dynamic class (extend PivotWidget) for each mdx result
            name = 'DbConfigurableWidget%s' % widget.id
            config, applied_filters = cwc.get_state(widget)
            requires = []
            for i in config['dimensions']:
                if i.has_key('fancy_select_module') and i['fancy_select_module'] not in requires:
                    requires.append(i['fancy_select_module'])

            if config.has_key('icon'):
                icon = config['icon']
            else:
                icon = ConfigurableWidget.icon

            cls = type(name, (DbConfigurableWidget,), {
                'label': widget.title,
                'category': widget.category,
                'requires': requires,
                'app': widget.application,
                'icon': icon,
                'params': {
                    'datasource_widget_id': widget.id,
                    'config': config,
                    'appliedConfig': applied_filters,
                    'graphConfig': config.has_key('graph_config') and config['graph_config'] or {},
                    'tableConfig': {'isTransposed': False, 'slicersExpanded': False}
                }
            })
            widgets.update([(name, cls)])
    return widgets