# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.
# this module is used for checking if the current user needs to flush the datasources he has access to

from iqportal.iqube.core.controllers import DataSourceController
from django.utils.translation import ugettext as _

class StaleDatasources:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.dsc = DataSourceController(self.sa_session, account_id=request.session['user_id'])

    def get_growls(self):
        """
        Returns a list of growl configs in case the user needs to see any notifications
        """
        growls = []
        if not self.dsc.has_permissions('update_datasource'):
            return growls

        datasources = self.dsc.get_datasources()
        for d in datasources:
            if d.has_stale_definitions:
                growl = {
                    'title': _('Stale datasource'),
                    'message': _("Datasource %s is stale.") % d.name,
                    'lifetime': 0,
                    'type': 'warning',
                    'callback_config': {
                        'map_key': 'stale_datasource',
                        'params': [d.id, d.name]
                    }
                }

                growls.append(growl)

        return growls

growlers = (StaleDatasources, )
