# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.contrib.iconbar.models import IconBarIcon

class IQubeDataSourceIcon(IconBarIcon):
    """
    Class responsible for handling the iqreport compose icon
    """

    id = 'icon-datasource-manager'
    title = _('Data source manager')
    icon_index = 44
    component = 'iqube_datasource_manager'
    group = 'iqube'

    char_main = u'\uE826' # Database icon
    char_sub = u'\uE806' # Man icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('view_datasource', 'datasource_manager')

class IQubeConfigurableWidgetManager(IconBarIcon):
    id = 'icon-configurable-widget-manager'
    title = _('Configurable widget manager')
    icon_index = 48
    component = 'iqube_configurable_widget_manager'
    group = 'iqube'

    char_main = u'\uE806' # Man icon
    char_sub = u'\uE80d' # Gear icon

    @assert_user_logged_in
    def assert_permissions(self, controller):
        """
        this is shown only if the permissions are met
        """
        controller.assert_permissions('create_datasource_widget')

def get_icons():
    """
    return the list of icons that the home app adds to the iconbar
    """

    return [IQubeDataSourceIcon(), IQubeConfigurableWidgetManager()]
