# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy.sql.expression import and_

from basesoft.scheduler.task import ScheduleTask
from basesoft import logging

from iqportal.iqube.core.models import DataSource
from iqportal.iqube.core.controllers import IQubeController,\
    DataSourceController
from sqlalchemy.orm.exc import NoResultFound

logger = logging.getLogger('iqportal.iqube.core')

class AbstractAggregateTask(ScheduleTask):
    """
    Abstract Task that can be used for populating an aggregated table and automagic clearing of the mondrian cache

    TODO: would be nice to be able to set an hour (or multiple) in which the task runs (when needed) #3256
    """

    INTERVAL = 3600 # one hour

    # override this list and specify a list of datasources that
    # get refreshed after the populate() call
    refresh_datasources = ()

    # if set false the cache wont clear all mondrian cache for its projects
    # in that case the subclass should call clear_mondrian_member_cache() itself (like in tnt)
    clear_mondrian_cache = True

    def __init__(self, process_id=None, start_time=None, params=None):
        ScheduleTask.__init__(self, process_id, start_time, params) # ScheduleTask is not a new-style class, so super doesnt work
        self._aggregators = []

    def execute(self, refresh=True):

        try:

            # call populate corben
            data_changed = self.populate()

            # expire local cache and clear mondrian cache on datasource(s) only when populate() did something
            if refresh and data_changed:
                ic = IQubeController()
                for ds_name in self.refresh_datasources:

                    try:
                        project = self.sa_session.query(DataSource).filter(and_(DataSource.name==ds_name)).one()
                    except NoResultFound:
                        raise Exception('datasource not found: %s' % ds_name)

                    # always clear all xmla cache
                    ic.clear_xmla_cache(project.mondrian_project, 0)

                    # if set also clear mondrian cache
                    if self.clear_mondrian_cache:
                        ic.clear_mondrian_cache(project.mondrian_project)
        finally:
            self.release()

    def release(self):

        for a in self._aggregators:
            a.release()

    def populate(self):
        """
        should return something that is 'true' that will be reported to the scheduler and only then the datasources will reload
        """
        raise NotImplementedError()

