# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from xml.sax.handler import ContentHandler

from iqportal.iqube.xmla.result import MdxResultProxy
from basesoft.web.core.exceptions import PublicUserException

class ExecuteHandler(ContentHandler):
    """
    Execute Handler for sax, used to create an MdxResult object.
    """
    processor = None
    def __init__(self):
        self.proxy = MdxResultProxy()

    def startElementNS(self, name, qname, attr):
        localname = name[1]

        if self.processor is not None:
            self.processor.process_child(localname, attr)

        processor = self.proxy.get_processor(localname, attr)
        if processor is not None:
            self.processor = processor

    def endElementNS(self, name, qname):
        if self.processor is not None and self.processor.tag == name[1]:
            self.processor = None

    def characters(self, data):
        if self.processor is not None:
            self.processor.set_data(data)

    def get_result(self):
        return self.proxy.get_result()

    def __repr__(self):
        return '<ExecuteHandler>'

class DiscoverHandler(ContentHandler):
    """
    Simple sax handler object to parse "row" tags from the XMLA provider's response
    """
    current_row = None
    current_item = None

    is_error= False
    error_code = None
    error_desc = None

    def __init__(self):
        self.result = []

    def startElementNS(self, name, qname, attr):

        localname = name[1]

        if localname == 'error':
            self.is_error = True

        if localname == 'row':
            self.current_row = {}
        elif self.current_row is not None:
            self.current_item = localname

    def characters(self, data):

        if self.current_item is not None:
            self.current_row[self.current_item.lower()] = data
            self.current_item = None

        self._current_data = data

    def endElementNS(self, name, qname):
        localname = name[1]

        if self.current_row is not None and localname == 'row':
            self.result.append(self.current_row)
            self.current_row = None

        if self.is_error:

            if localname == 'code':
                self.error_code = self._current_data

            if localname == 'desc':
                self.error_desc = self._current_data

    def get_result(self):

        if self.is_error:
            e = XmlaException(self.error_desc)
            e.code = self.error_code
            raise e

        return self.result

    def __repr__(self):
        return '<DiscoverHandler result="%s">' % self.result


class XmlaException(PublicUserException):
    code = None

