# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from iqportal.iqube.core.db_sync import AbstractConfigurableWidget
from iqportal.iqube.core.models import DataSourceWidget

from django.utils.translation import ugettext as _

class DefaultQAItemWidget(AbstractConfigurableWidget):
    category = 'IQ Report - IQuest'
    cube = 'VirtualResponse'
    datasource = 'IQuest'
    label = _("Default QA Item")

    availability = DataSourceWidget.AVAILABLE_AS_DASHBOARD_WIDGET+DataSourceWidget.AVAILABLE_AS_REPORT_ITEM

    def config(self):
        return {
            "icon": "16x16/list.png",
            "category": "IQuest",
            "show_hide_empty": True,
            "transposable": True,
            "has_quick_filter": True,
            "dimensions": [
                {
                    "available_on_rows": False,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": True,
                    "required": True,
                    "available_as_slicer": True,
                    "caption": "Survey",
                    "fancy_select_module": "level_filter_fancy_select",
                    "available_on_columns": False,
                    "show_calculated_members": False,
                    "children": [
                        {
                            "unique_name": "[Survey].[Survey]",
                            "has_children_available": False,
                            "caption": "Survey",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Survey].[Survey]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Survey].[Publication]",
                            "has_children_available": False,
                            "caption": "Publication",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Survey].[Publication]",
                            "method": "get_level_members"
                        }
                    ]
                },
                {
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": False,
                    "available_as_split_by": True,
                    "required": False,
                    "available_as_slicer": False,
                    "caption": "Question",
                    "fancy_select_module": "level_filter_fancy_select",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                    "children": [
                        {
                            "unique_name": "[Question].[Item]",
                            "has_children_available": False,
                            "caption": "Item",
                            "has_all_member_available": False,
                            "show_members": False,
                            "method_argument": "[Question].[Item]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Question].[Locale]",
                            "has_children_available": False,
                            "caption": "Locale",
                            "has_all_member_available": False,
                            "show_members": False,
                            "method_argument": "[Question].[Locale]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Question].[Label]",
                            "has_children_available": False,
                            "caption": "Label",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Question].[Label]",
                            "method": "get_level_members"
                        }
                    ]
                },
                {
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": False,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": False,
                    "caption": "Answer",
                    "fancy_select_module": "level_filter_fancy_select",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                    "children": [
                        {
                            "unique_name": "[Answer].[Item]",
                            "has_children_available": False,
                            "caption": "Item",
                            "has_all_member_available": False,
                            "show_members": False,
                            "method_argument": "[Answer].[Item]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Answer].[Locale]",
                            "has_children_available": False,
                            "caption": "Locale",
                            "has_all_member_available": False,
                            "show_members": False,
                            "method_argument": "[Answer].[Locale]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Answer].[Label]",
                            "has_children_available": False,
                            "caption": "Label",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Answer].[Label]",
                            "method": "get_level_members"
                        }
                    ]
                },
                {
                    "available_on_rows": False,
                    "show_named_sets": False,
                    "has_select_as_group": False,
                    "available_as_split_by": False,
                    "required": True,
                    "available_as_slicer": True,
                    "caption": "Measures",
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": False,
                    "show_calculated_members": False,
                    "children": [
                        {
                            "unique_name": "[Measures].[MeasuresLevel]",
                            "has_children_available": False,
                            "caption": "MeasuresLevel",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Measures].[MeasuresLevel]",
                            "method": "get_level_members"
                        }
                    ]
                },
            ],
            "reporter": "iqportal.iquest.report.DefaultQAItemReporter",
            "has_drill_through": False,
            "datasource_id": 1,
            "label": "Default QA Item",
            "show_excel_export": True,
            "has_drill_position": False,
            "cube": "Response",
            "has_graph": True,
            "has_calculated_member": True,
            "id": 6,
            "table_object_name": "iqube_graph_datatablecube",
            "table_module": "iqube_graph_datatablecube",
            "has_table": True,
            "graph_config": {},
            "has_drill_replace": False,
            'filter_panel_module': 'iquest_default_qa_report_item',
            'global_filters_module': 'iquest_global_report_filters',
        }

    def applied_filters(self):
        return {
            "axes_0": "slicers",
            "axes_1": "rows",
            "axes_2": "columns",
            "axes_3": "slicers",
            "dimensions": {
                "Survey": "0",
                "Question": "1",
                "Answer": "2",
                "Measures": "3",
            },
            "column_order": [
                "Answer"
            ],
            "row_order": [
                "Question"
            ],
            "show_table": True,
            "dimension_0": [],
            "dimension_1": [],
            "dimension_2": [],
            "dimension_3": [
                "[Measures].[Responses Int Sum]",
                "[Measures].[Responses Float Sum]",
            ],
            'initialRefresh': False,
        }
