# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from iqportal.iqube.core.db_sync import AbstractConfigurableWidget
from iqportal.iqube.core.models import DataSourceWidget

from django.utils.translation import ugettext as _

class PublicationStatisticsItemWidget(AbstractConfigurableWidget):
    category = 'IQ Report - IQuest'
    cube = 'PublicationStatistics'
    datasource = 'IQuest'
    label = _("Publication statistics")
    availability = DataSourceWidget.AVAILABLE_AS_DASHBOARD_WIDGET+DataSourceWidget.AVAILABLE_AS_REPORT_ITEM

    def config(self):
        return {
            "icon": "16x16/list.png",
            "category": "IQuest",
            "show_hide_empty": True,
            "transposable": True,
            "dimensions": [
                {
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": True,
                    "available_as_split_by": True,
                    "required": True,
                    "available_as_slicer": True,
                    "caption": "Survey",
                    "fancy_select_module": "level_filter_fancy_select",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                    "children": [
                        {
                            "unique_name": "[Survey].[Survey]",
                            "has_children_available": False,
                            "caption": "Survey",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Survey].[Survey]",
                            "method": "get_level_members"
                        },
                        {
                            "unique_name": "[Survey].[Publication]",
                            "has_children_available": False,
                            "caption": "Publication",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Survey].[Publication]",
                            "method": "get_level_members"
                        }
                    ]
                },
                {
                    "available_on_rows": True,
                    "show_named_sets": False,
                    "has_select_as_group": False,
                    "available_as_split_by": False,
                    "required": False,
                    "available_as_slicer": True,
                    "caption": "Measures",
                    "fancy_select_module": "iq_fancyselect",
                    "available_on_columns": True,
                    "show_calculated_members": False,
                    "children": [
                        {
                            "unique_name": "[Measures].[MeasuresLevel]",
                            "has_children_available": False,
                            "caption": "MeasuresLevel",
                            "has_all_member_available": False,
                            "show_members": True,
                            "method_argument": "[Measures].[MeasuresLevel]",
                            "method": "get_level_members"
                        }
                    ]
                },
            ],
            "reporter": "iqportal.iquest.report.PublicationStatisticsReporter",
            "has_drill_through": False,
            "datasource_id": 1,
            "label": "Publication statistics",
            "show_excel_export": True,
            "has_drill_position": False,
            "cube": "PublicationStatistics",
            "has_graph": True,
            "id": 6,
            "table_object_name": "iqube_graph_datatablecube",
            "table_module": "iqube_graph_datatablecube",
            "has_table": True,
            "graph_config": {},
            "has_drill_replace": False,
            "has_calculated_member": True,
            'filter_panel_module': 'iquest_publication_statistics_item',
            'global_filters_module': 'iquest_global_report_filters',
        }

    def applied_filters(self):
        return {
            "axes_0": "rows",
            "axes_1": "columns",
            "dimensions": {
                "Survey": "0",
                "Measures": "1",
            },
            "column_order": [
                "Measures"
            ],
            "row_order": [
                "Survey"
            ],
            "show_table": True,
            "dimension_0": [],
            "dimension_1": [
                "[Measures].[PubStats Invitees]",
                "[Measures].[PubStats Respondents]",
            ],
            'initialRefresh': False,
        }
