# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.utils import simplejson
from django.utils.translation import ugettext as _

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.contrib.i18n import models as i18n_models

from iqportal.iquest.compose.controllers import ComposeController
from iqportal.iquest.report.controllers import ReportController
from iqportal.iquest.core.models import SurveyPublication
from basesoft.web.contrib.compose.views import compose_runner
from basesoft.web.contrib.license.controllers import LicenseController

def jsonrpc_view(request):
    """
    """
    class RPCHandler:

        def __init__(self, sa_session, account_id):
            self.sa_session = sa_session
            self.account_id = account_id
            locale = self.sa_session.query(i18n_models.Locale).filter_by(locale='en_GB').one()
            self.cc = ComposeController(self.sa_session, locale=locale, account_id=self.account_id)

        def get_publication_info(self, publication_id):
            """
            """
            assert type(publication_id) == int

            publication = self.cc.get_publications(publication_id).pop()

            from iqportal.iquest.response.controllers import PublicationResponseController
            controller = PublicationResponseController(self.sa_session, account_id=self.account_id)

            details = controller.get_publication_details(publication_id)

            if publication.anonymous:
                invitees = _('Anonymous')
            else:
                invitees = details['invitees']

            return {
                'status': SurveyPublication.status_i18n[publication.status],
                'begin_date': publication.begin_date,
                'end_date': publication.end_date,
                'number_of_questions': len(publication.survey.itemQAs),

                'locales': [l.name for l in publication.survey.available_locales],
                'invitees': invitees,
                'participants': details['participants'],
                'respondents': details['respondents']
            }

        def load_report_state(self, publication_id):
            """
            """
            publication = self.cc.get_publications(publication_id).pop()

            rc = ReportController(self.sa_session, account_id=self.account_id)
            return rc.get_default_report_state(publication)


    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(RPCHandler(request.Session(), request.session['user_id']))
    return dispatcher.handle_request(request)

def default_report(request, params_str):
    """
    called by the iqreport preview panel in an iframe
    """

    vars_ = simplejson.loads(params_str)

    locale = request.session['config']['basesoft.web.contrib.locale']
    account_id = request.session['user_id']

    c = ComposeController(request.Session(), locale, account_id)
    pub = c.get_publications(vars_['runner_id'], priv=SurveyPublication.PRIV_VIEW_RESPONSE)[0]


    if vars_.has_key('publication_id') and vars_['publication_id'] is not None:
        # get the publication so we can correctly load the cache for the report.
        p = c.get_report_publication(vars_['publication_id']) # this validates view privs on the publication
        vars_['publication_label'] = p.label
        request.session['report_publication_id'] = p.id
    elif request.session.has_key('report_publication_id'):
        del request.session['report_publication_id']

    lc = LicenseController(request.Session())
    site_licensee = lc.get_license_for_request(request)
    licensee = site_licensee.http_host

    requires = ('iquest_report_init', )
    title = '%s - %s - %s' % ( pub.survey.name, pub.label, _('Default report') )
    vars_['compose_name'] = '%s - %s - %s' % ( pub.survey.name, pub.label, _('Default report') )

    # if an invite previously opened its own report, the viewer's id was stored in session
    # and now on preview that one will be used without any privs checks
    # so make sure a viewer id is not used when opening a report preview
    if request.session.has_key('report_viewer_id'):
        del request.session['report_viewer_id']

    return compose_runner(request, requires, vars_, title, licensee)

