# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from iqportal.sap.core import models as sap_models
from basesoft.web.contrib.license.models import Domain
from sqlalchemy.orm.exc import NoResultFound

def sync(session):
    sap_domain = None
    try:
        sap_domain = session.query(Domain).filter(Domain.name=='SAP').one()
    except NoResultFound:
        print 'SAP core::db_sync: domain SAP not found, skipping...'
        return

    type_objs = {}
    types = ['Component', 'Function']
    for t in types:
        tt = session.query(sap_models.SAPComponentElementType).filter(sap_models.SAPComponentElementType.name==t).all()
        if len(tt) > 0:
            type_objs[t] = tt[0]
        else:
            tt = sap_models.SAPComponentElementType()
            tt.name = t
            session.add(tt)
            type_objs[t] = tt
    session.flush()

    components = [
        # ------ Components
        {'type': 'Component', 'name': 'Purchasing (MM-PUR)', 'code': 'MM100200', 'parent': None},
        {'type': 'Component', 'name': 'Interfacing', 'code': 'SAPIP100100', 'parent': None},
        {'type': 'Component', 'name': 'Contract Management', 'code': 'SRM100300', 'parent': None},
        {'type': 'Component', 'name': 'Collaborative Procurement', 'code': 'SRM100400', 'parent': None},
        {'type': 'Component', 'name': 'Supplier Collaboration', 'code': 'SRM100500', 'parent': None},

        # ------ Functions
        {'type': 'Function', 'name': 'Purchasing Menu', 'code': 'MM200300', 'parent': 'Purchasing (MM-PUR)'},
        {'type': 'Function', 'name': 'Purchasing Document', 'code': 'MM200400', 'parent': 'Purchasing (MM-PUR)'},
        {'type': 'Function', 'name': 'Master Records from the Purchasing View', 'code': 'MM200500', 'parent': 'Purchasing (MM-PUR)'},

        {'type': 'Function', 'name': 'Interfacing MM-FICO', 'code': 'SAPIP200100', 'parent': 'Interfacing'},
        {'type': 'Function', 'name': 'Interfacing MM SAP PS', 'code': 'SAPIP200400', 'parent': 'Interfacing'},

        {'type': 'Function', 'name': 'Contract Execution', 'code': 'SRM201000', 'parent': 'Contract Management'},
        {'type': 'Function', 'name': 'Contract Monitoring', 'code': 'SRM201100', 'parent': 'Contract Management'},

        {'type': 'Function', 'name': 'Self-Service Procurement', 'code': 'SRM201200', 'parent': 'Collaborative Procurement'},
        {'type': 'Function', 'name': 'Catalog Content Management', 'code': 'SRM201500', 'parent': 'Collaborative Procurement'},

        {'type': 'Function', 'name': 'Direct Document Exchange', 'code': 'SRM201700', 'parent': 'Supplier Collaboration'}
    ]

    # add privileges for sap
    group = sap_domain.default_group

    component_objs = {}
    for component in components:
        g = session.query(sap_models.SAPComponentElement).filter(sap_models.SAPComponentElement.name==component['name']).all()
        if len(g) > 0:
            component_objs[component['name']] = g[0]
            continue
        if not type_objs.has_key(component['type']):
            raise Exception('Invalid type %s' % component['type'])

        g = sap_models.SAPComponentElement()
        g.name = component['name']
        g.code = component['code']
        g.type = type_objs[component['type']]

        if component.has_key('parent') and component['parent'] is not None:
            if not component_objs.has_key(component['parent']):
                raise Exception('Invalid parent %s' % component['parent'])
            g.parent_component_element = component_objs[component['parent']]

        object = g.get_group_object()[0]
        for privilege in g.all_privileges:
            go = object()
            go.group = group
            go.privilege = privilege
            go.group_has_privilege = True
            go.license_has_privilege = True
            session.add(go)
            g.groups.append(go)

        session.add(g)
        component_objs[component['name']] = g

    session.flush()