# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.
from iqportal.xbrl.core.models import XBRLProject, XBRLElement

from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy import and_
from django.conf import settings
from iqportal.iqube.core.models import DataSource, DataSourceCalculatedMember
from basesoft.web.contrib.license.models import Domain
from iqportal.iqube.core.db_sync import AbstractConfigurableWidget

def sync(session):

    # see if the Annual report is created
    try:
        project = session.query(XBRLProject).filter(XBRLProject.name == 'DutchAnnualReports').one()
    except NoResultFound:
        print 'No XBRL Project found for the Dutch Annual Report. Skipping sync...'
        return

    translate_elements(session, project)

    domain = session.query(Domain).filter(Domain.name=='Resellers').one()
    group = domain.default_group

    ds = session.query(DataSource).filter(DataSource.name=='%s.XBRL' % settings.SETTINGS_MODULE).one()
    cube = 'XBRL'

    named_sets = (
        ('Assets, Non-Current', [
            'Property Plant and Equipment, Net',
            'Intangible Assets, Net',
            'Investments in Subsidiaries at Cost',
            'Deferred Tax Assets',
            'Other Financial Assets, Non-Current',
            'Trade and Other Receivables, Net, Non-Current',
            'Other Assets, Non-Current',
            'Assets, Non-Current, Total'
        ]),
        ('Assets, Current', [
            'Non-Current Assets and Disposal Groups Held for Sale',
            'Inventories',
            'Trade and Other Receivables, Net, Current',
            'Cash and Cash Equivalents',
            'Other Assets, Current',
            'Assets, Current, Total',
        ]),
        ('Assets, Total', [
            'Assets, Total',
        ]),
        ('Equity Attributable to Equity Holders of Parent', [
            'Subscribed Capital',
            'Minority Interest',
            'Equity, Total',
        ]),
        ('Liabilities, Non-Current', [
            'Interest Bearing Borrowings, Non-Current',
            'Provisions, Non-Current',
            'Deferred Tax Liabilities',
            'Other Liabilities, Non-Current',
            'Liabilities, Non-Current, Total',
        ]),
        ('Liabilities, Current', [
            'Liabilities Included in Disposal Groups Classified as Held for Sale',
            'Interest Bearing Borrowings, Current',
            'Provisions, Current',
            'Trade and Other Payables, Current',
            'Accrued Liabilities, Total',
            'Other Liabilities, Current',
            'Liabilities, Current, Total',
        ]),
        ('Liabilities, Total', [
            'Liabilities, Total',
        ]),
        ('Equity and Liabilities, Total', [
            'Equity and Liabilities, Total',
        ]),
        ('Gross Profit by Function', [
            'Revenue, Total by Function',
            'Cost of Sales by Function',
            'Gross Profit by Function',
        ]),
        ('Other Operating Income by Function', [
            'Miscellaneous Other Operating Income by Function',
            'Other Operating Income, Total by Function'
        ]),
        ('Operating Expenses by Function', [
            'Marketing and Distribution Costs by Function',
            'Research and Development',
            'Administrative Expenses by Function',
            'Restructuring Costs',
            'Miscellaneous Other Operating Expenses by Function',
            'Operating Expenses, Total by Function',
            'Profit (Loss) from Operations',
        ]),
        ('Profit (Loss) Before Tax', [
            'Finance Costs for Non Financial Activities',
            'Share of Profit (Loss) from Equity Accounted Investments',
            'Profit (Loss) Before Tax',
        ]),
        ('Profit (Loss) After Tax from Continuing Operations', [
            'Income Tax Expense Income',
            'Profit (Loss) After Tax from Continuing Operations',
        ]),
        ('Profit (Loss)', [
            'Profit (Loss) from Discontinued Operations, Net of Tax',
            'Profit (Loss)',
        ]),
        ('Profit (Loss) Attributable to Equity Holders of Parent and Minority Interest', [
            'Profit (Loss) Attributable to Equity Holders of Parent',
            'Profit (Loss) Attributable to Minority Interest',
        ])
    )

    current_sets = session.query(DataSourceCalculatedMember).filter(
        and_(
             DataSourceCalculatedMember.datasource==ds,
             DataSourceCalculatedMember.cube==cube,
             DataSourceCalculatedMember.dimension=='[ProjectElement]',
             DataSourceCalculatedMember.type==DataSourceCalculatedMember.CALCULATED_SET
        )).all()
    names = {}
    for s in current_sets:
        names[s.name] = s

    for set in named_sets:
        name, members = set
        if names.has_key(name):
            cm = names[name]
        else:
            cm = DataSourceCalculatedMember()
            cm.datasource = ds
            cm.cube = cube
            cm.dimension = '[ProjectElement]'
            cm.type=DataSourceCalculatedMember.CALCULATED_SET

        cm.name = name
        cm.formula = '{%s}' % ','.join(['[ProjectElement].[DutchAnnualReports].[%s]' % m for m in members])
        if cm.id is None:
            object = cm.get_group_object()[0]
            for privilege in cm.all_privileges:
                go = object()
                go.group = group
                go.privilege = privilege
                go.group_has_privilege = True
                go.license_has_privilege = True
                session.add(go)
                cm.groups.append(go)
        session.add(cm)
        session.flush()
        names[name] = cm

class DefaultXBRLWidget(AbstractConfigurableWidget):
    category = 'XBRL'
    cube = 'XBRL'
    datasource = 'XBRL'

    # these widgets need the named sets to be in the Database.
    requires_app_sync = 'iqportal.xbrl.report'

    def _get_dimension_config(self, hierarchy_caption, level_caption, show_members=True):
        return {
            u'fancy_select_module': 'iq_fancyselect',
            u'available_on_rows': False,
            u'show_named_sets': False,
            u'has_select_as_group': False,
            u'available_as_split_by': False,
            u'required': True,
            u'available_as_slicer': True,
            u'caption': hierarchy_caption,
            u'available_on_columns': False,
            u'show_calculated_members': False,
            u'children': [{
                u'show_members': show_members,
                u'method_argument': u'[%s].[%s]' % (hierarchy_caption, level_caption),
                u'caption': level_caption,
                u'unique_name': u'[%s].[%s]' % (hierarchy_caption, level_caption),
                u'method': u'get_level_members'
            }]
        }

    def config(self):
        return {
            u'show_excel_export': False,
            u'table_module': u'xbrl_graph_datatablecube',
            u'table_object_name': u'xbrl_graph_datatablecube',
            u'reporter': u'iqportal.xbrl.report.DutchAnnualReporter',
            u'show_hide_empty': True,
            u'has_graph': True,
            u'transposable': False,
            u'label': u'Yearly Report',
            u'has_table': True,
            u'dimensions': [
                self._get_dimension_config('EntityIdentifier', 'Identifier'),
                self._get_dimension_config('Measures', 'MeasuresLevel'),
                self._get_dimension_config('ProjectElement', 'Element', False),
                self._get_dimension_config('Time', 'Year'),
            ]}

    def applied_filters(self):
        return {
            u'axes_0': u'slicers',
            u'axes_1': u'slicers',
            u'axes_2': u'rows',
            u'axes_3': u'columns',
            u'dimensions': {
                u'EntityIdentifier': u'0',
                u'ProjectElement': u'2',
                u'Measures': u'1',
                u'Time': u'3'
            },
            u'show_table': True,
            u'split_by_2': True,
            u'dimension_0': [u'[EntityIdentifier].[17001910]'],
            u'dimension_1': [u'[Measures].[FloatData]'],
            u'dimension_2': self.get_named_sets(),
            u'dimension_3': [u'[Time].[2004]', u'[Time].[2005]'],
            u'hide_empty_fields': True
        }

    def get_named_sets(self, sets):
        return [cm.id for cm in self.sa_session.query(DataSourceCalculatedMember).filter(
        and_(
             DataSourceCalculatedMember.datasource==self.get_datasource(),
             DataSourceCalculatedMember.cube==self.cube,
             DataSourceCalculatedMember.dimension=='[ProjectElement]',
             DataSourceCalculatedMember.type==DataSourceCalculatedMember.CALCULATED_SET,
             DataSourceCalculatedMember.name.in_(sets)
        )).all()]

    def validate_requirements(self):
        project = self.sa_session.query(XBRLProject).filter(XBRLProject.name == 'DutchAnnualReports').first()
        if project is None:
            return False
        return True

class BalanceSheetWidget(DefaultXBRLWidget):
    label = 'Balance Sheet'

    def get_named_sets(self):
        sets = [
            'Assets, Non-Current',
            'Assets, Current',
            'Assets, Total',
            'Equity Attributable to Equity Holders of Parent',
            'Liabilities, Non-Current',
            'Liabilities, Current',
            'Liabilities, Total',
            'Equity and Liabilities, Total',
        ]
        return DefaultXBRLWidget.get_named_sets(self, sets)

class IncomeStatementWidget(DefaultXBRLWidget):
    label = 'Income Statement'

    def get_named_sets(self):
        sets = [
            'Gross Profit by Function',
            'Other Operating Income by Function',
            'Operating Expenses by Function',
            'Profit (Loss) Before Tax',
            'Profit (Loss) After Tax from Continuing Operations',
            'Profit (Loss)',
            'Profit (Loss) Attributable to Equity Holders of Parent and Minority Interest',

        ]
        return DefaultXBRLWidget.get_named_sets(self, sets)

configurable_widgets = (BalanceSheetWidget, IncomeStatementWidget, )

def translate_elements(session, project):
    import re
    elements = session.query(XBRLElement).filter(XBRLElement.project==project).all()

    all_cap_re = re.compile('([a-z0-9])([A-Z])')
    def convert(name):
        lower = ['In', 'For', 'And', 'At', 'Of', 'By', 'As', 'From']
        rv = all_cap_re.sub(r'\1 \2', name)
        for i in lower:
            rv = rv.replace(' %s ' % i, ' %s ' % i.lower())

        rv = rv.replace('Non Current', 'Non-Current')
        rv = rv.replace(' Non-Current', ', Non-Current')
        rv = rv.replace(' Current', ', Current')
        rv = rv.replace(' Net', ', Net')
        rv = rv.replace(' Total', ', Total')
        rv = rv.replace('Profit Loss', 'Profit (Loss)')

        return rv

    for element in elements:
        c = convert(element.name)
        if element.handle == c:
            continue
        element.handle = c
        session.add(element)

    session.flush()




