# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from iqportal.iqube.core.models import DataSource
from basesoft.web.core.management.commands.iqubesync import ProjectBase, CubeBase

from django.conf import settings
"""
# EXAMPLE MDX Growth (project annual report)

with member [Time].[Growth] as
'(([Measures].[FloatData], [Time].[2005]) - ([Measures].[FloatData], [Time].[2004]))/([Measures].[FloatData],
[Time].[2004])', FORMAT_STRING='0.00%'

SELECT NON EMPTY CROSSJOIN({[Measures].[FloatData]}, {[Time].[2004], [Time].[2005], [Time].[Growth]}) ON COLUMNS,
    NON EMPTY {[ProjectElement].[DutchAnnualReports].Children} ON ROWS
FROM [XBRL]
WHERE {[EntityIdentifier].[All Identifiers].[17001910]}
"""

class XBRLProject(ProjectBase):
    project_name = 'XBRL'
    schema_name = 'XBRL'
    description = 'XBRL project'
    world_privileges = DataSource.PRIV_VIEW

    def __init__(self, sa_session, controller):
        ProjectBase.__init__(self, sa_session, controller)
        self._db_from_connstr(settings.DB_CONNSTR_READONLY)

    def get_cubes(self):
        return (XBRLCube, )

class XBRLCube(CubeBase):
    name = 'XBRL'

    xml = """
    <cube name="XBRL">
        <Table name="tblXBRLFact" />

        <Dimension name="ProjectElement" foreignKey="element_id">
            <Hierarchy hasAll="False" primaryKey="id" primaryKeyTable="tblXBRLElement">
                <Join leftKey="project_id" rightKey="id">
                    <Table name="tblXBRLElement" />
                    <Table name="tblXBRLProject" />
                </Join>
                <Level name="Project" column="name" table="tblXBRLProject" uniqueMembers="true" />
                <Level name="Element" column="handle" table="tblXBRLElement" uniqueMembers="true" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Unit" foreignKey="unit_id">
            <Hierarchy hasAll="True" primaryKey="id" allMemberName="All Units">
                <Table name="tblXBRLUnit" />
                <Level name="Measure" column="measure" table="tblXBRLUnit" uniqueMembers="false" />
            </Hierarchy>
        </Dimension>

        <Dimension name="EntityIdentifier" foreignKey="context_id">
            <Hierarchy hasAll="True" primaryKey="id" primaryKeyTable="tblXBRLContext" allMemberName="All Identifiers">
                <Join leftKey="entity_id" rightKey="id">
                    <Table name="tblXBRLContext" />
                    <Join leftKey="identifier_id" rightKey="id">
                        <Table name="tblXBRLContextEntity" />
                        <Table name="tblXBRLContextEntityIdentifier" />
                    </Join>
                </Join>
                <Level name="Identifier" column="identifier" table="tblXBRLContextEntityIdentifier" uniqueMembers="true" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Time" type="TimeDimension" foreignKey="context_id">
            <Hierarchy hasAll="False" primaryKey="id" primaryKeyTable="tblXBRLContext">
                <Join leftKey="period_id" rightKey="id">
                    <Table name="tblXBRLContext" />
                    <Table name="tblXBRLPeriod" />
                </Join>
                <Level name="Year" column="the_year" table="tblXBRLPeriod" type="Numeric" uniqueMembers="true" levelType="TimeYears"/>
                <Level name="Quarter" column="quarter" table="tblXBRLPeriod" uniqueMembers="false" levelType="TimeQuarters"/>
                <Level name="Month" column="month_of_year" table="tblXBRLPeriod" uniqueMembers="false" type="Numeric" levelType="TimeMonths"/>
            </Hierarchy>

            <Hierarchy hasAll="true" name="Weekly" primaryKey="id" primaryKeyTable="tblXBRLContext">
                <Join leftKey="period_id" rightKey="id">
                    <Table name="tblXBRLContext" />
                    <Table name="tblXBRLPeriod" />
                </Join>
                <Level name="Year" column="the_year" table="tblXBRLPeriod" type="Numeric" uniqueMembers="true" levelType="TimeYears"/>
                <Level name="Week" column="week_of_year" table="tblXBRLPeriod" type="Numeric" uniqueMembers="false" levelType="TimeWeeks"/>
                <Level name="Day" column="day_of_month" table="tblXBRLPeriod" uniqueMembers="false" type="Numeric" levelType="TimeDays"/>
            </Hierarchy>
        </Dimension>

       <Dimension name="Segment" foreignKey="context_id">
            <Hierarchy hasAll="True" primaryKey="id" primaryKeyTable="tblXBRLContext" allMemberName="All Segments">
                <Join leftKey="entity_id" rightKey="id">
                    <Table name="tblXBRLContext" />
                    <Join leftKey="id" rightKey="entity_id">
                        <Table name="tblXBRLContextEntity" />
                        <Join leftKey="segment_id" rightKey="id">
                            <Table name="tblXBRLContextEntitySegment" />
                            <Table name="tblXBRLContextSegment" />
                        </Join>
                    </Join>
                </Join>
                <Level name="Dimension" column="dimension" table="tblXBRLContextSegment" uniqueMembers="true" />
                <Level name="Segment" column="name" table="tblXBRLContextSegment" uniqueMembers="false" />
            </Hierarchy>
        </Dimension>

        <Measure name="IntData" column="int_data" aggregator="sum" />
        <Measure name="FloatData" column="float_data" aggregator="sum" />
    </cube>
    """

if hasattr(settings, 'DB_CONNSTR_READONLY'):
    projects = (XBRLProject, )